__RCSID__ = "8f34a5d (2015-01-12 13:07:33 +0000) Xianghu Zhao <zhaoxh@ihep.ac.cn>"

from DIRAC                                            import S_OK, S_ERROR
from DIRAC.Core.Base.AgentModule                      import AgentModule
from DIRAC.Core.DISET.RPCClient                       import RPCClient

from DIRAC.WorkloadManagementSystem.DB.JobDB          import JobDB
from IHEPDIRAC.WorkloadManagementSystem.DB.TaskDB      import TaskDB

import time

class TaskAgent( AgentModule ):
  """
      The specific agents must provide the following methods:
      - initialize() for initial settings
      - beginExecution()
      - execute() - the main method called in the agent cycle
      - endExecution()
      - finalize() - the graceful exit of the method, this one is usually used
                 for the agent restart
  """

  def initialize( self ):
    self.__taskDB = TaskDB()
    self.__jobDB = JobDB()
    return S_OK()

  def execute( self ):
    """ Main execution method
    """
    condDict = { 'Status': ['Ready', 'Processing', 'Finished'] }
    result = self.__taskDB.getTasks( [ 'TaskID', 'Status' ], condDict )
    if not result['OK']:
      return result

    tasks = result['Value']

    self.log.info( '%d tasks will be refreshed' % len(tasks) )

    for task in tasks:
      taskID = task[0]
      status = task[1]

      if status in ['Ready', 'Processing', 'Finished']:
        self.__refreshTask( taskID )

    return S_OK()


  def __refreshTask( self, taskID ):
    result = self.__refreshTaskStringAttribute( taskID, 'Site' )
    if result['OK']:
      self.log.debug( 'Task %d site is refreshed' % taskID )
    else:
      self.log.error( 'Task %d site refresh failed: %s' % ( taskID, result['Message'] ) )

    result = self.__refreshTaskStringAttribute( taskID, 'JobGroup' )
    if result['OK']:
      self.log.debug( 'Task %d job group is refreshed' % taskID )
    else:
      self.log.error( 'Task %d job group refresh failed: %s' % ( taskID, result['Message'] ) )

    result = self.__refreshTaskStatus( taskID )
    if result['OK']:
      self.log.debug( 'Task %d status is refreshed' % taskID )
    else:
      self.log.error( 'Task %d status refresh failed: %s' % ( taskID, result['Message'] ) )


################################################################################

  def __getTaskProgress( self, taskID ):
    result = self.__taskDB.getTaskJobs( taskID )
    if not result['OK']:
      return result
    jobIDs = result['Value']

    result = self.__jobDB.getAttributesForJobList( jobIDs, ['Status'] )
    if not result['OK']:
      return result
    statuses = result['Value']

    progress = { 'Total': 0, 'Done': 0, 'Failed': 0, 'Running': 0, 'Waiting': 0, 'Deleted': 0 }
    progress['Total'] = len(jobIDs)
    for jobID in jobIDs:
      if jobID in statuses:
        status = statuses[jobID]['Status']
        if status in ['Done']:
          progress['Done'] += 1
        elif status in ['Failed', 'Stalled', 'Killed']:
          progress['Failed'] += 1
        elif status in ['Running', 'Completed']:
          progress['Running'] += 1
        else:
          progress['Waiting'] += 1
      else:
        progress['Deleted'] += 1

    return S_OK( progress )

  def __analyseTaskStatus( self, progress ):
    totalJob = progress.get( 'Total', 0 )
    runningJob = progress.get( 'Running', 0 )
    waitingJob = progress.get( 'Waiting', 0 )
    deletedJob = progress.get( 'Deleted', 0 )

    status = 'Unknown'
    if deletedJob == totalJob:
      status = 'Expired'
    elif runningJob == 0 and waitingJob == 0:
      status = 'Finished'
    else:
      status = 'Processing'

    return status

  def __refreshTaskStatus( self, taskID ):
    """ Refresh the task status
    """
    # get task progress from the job list
    result = self.__getTaskProgress( taskID )
    if not result['OK']:
      return result
    progress = result['Value']
    self.log.debug( 'Task %d Progress: %s' % ( taskID, progress ) )
    result = self.__taskDB.updateTaskProgress( taskID, progress )
    if not result['OK']:
      return result

    # get previous task status
    result = self.__taskDB.getTaskStatus( taskID )
    if not result['OK']:
      return result
    status = result['Value']

    # get current task status from the progress
    newStatus = self.__analyseTaskStatus( progress )
    self.log.debug( 'Task %d new status: %s' % ( taskID, newStatus ) )
    if newStatus != status:
      self.__taskDB.updateTaskStatus( taskID, newStatus, 'Status refreshed' )
      if not result['OK']:
        return result

    return S_OK( newStatus )


################################################################################

  def __getTaskAttribute( self, taskID, attributeType ):
    """ Get all attributes of the jobs in the task
    """
    result = self.__taskDB.getTaskJobs( taskID )
    if not result['OK']:
      return result
    jobIDs = result['Value']

    condDict = { 'JobID': jobIDs }

    result = self.__jobDB.getDistinctJobAttributes( attributeType, condDict )
    if not result['OK']:
      return result
    attributes = result['Value']

    return S_OK( attributes )

  def __refreshTaskStringAttribute( self, taskID, attributeType ):
    """ Refresh the task attribute. The attribute type must be string and seperated by comma
    """
    # get task attibutes from the job list
    result = self.__getTaskAttribute( taskID, attributeType )
    if not result['OK']:
      return result
    newAttributes = result['Value']

    # get previous task attributes
    result = self.__taskDB.getTask( taskID, [attributeType] )
    if not result['OK']:
      return result
    oldAttributes = result['Value'][0].split( ',' )

    # check whether there are differences
    if set( newAttributes ) == set( oldAttributes ):
      self.log.debug( 'Task %s attribute is the same: %s' % (attributeType, oldAttributes) )
      return S_OK( oldAttributes )

    # make a combination of old and new attributes
    attributes = list( set( oldAttributes ) | set( newAttributes ) )
    for emptyAttr in [ '', 'ANY', 'Multiple' ]:
      if emptyAttr in attributes:
        attributes.remove( emptyAttr )

    # generate a new attribute
    allAttributes = ','.join( attributes )
    result = self.__taskDB.updateTask( taskID, [attributeType], [allAttributes] )
    if not result['OK']:
      return result

    return S_OK( allAttributes )




from DIRAC                                import S_OK, S_ERROR
from DIRAC.Core.DISET.RPCClient           import RPCClient

class TaskClient( object ):
  """
      The task manager client
  """

  def __init__( self ):
    self.__taskManager = RPCClient('WorkloadManagement/TaskManager')
    self.__jobManager = RPCClient('WorkloadManagement/JobManager')
    self.__jobMonitor = RPCClient('WorkloadManagement/JobMonitoring')


  def createTask( self, taskName, taskInfo, jobInfos ):
    return self.__taskManager.createTask( taskName, taskInfo, jobInfos )


  def getTaskCount( self, condDict ):
    return self.__taskManager.getTaskCount( condDict )

  def listTask( self, condDict, limit, offset, orderAttribute ):
    return self.__taskManager.getTasks( condDict, limit, offset, orderAttribute, 1 )


  def renameTask( self, taskID, newName ):
    return self.__taskManager.renameTask( taskID, newName )

  def getTaskOwners( self ):
    return self.__taskManager.getTaskOwners()

  def getTaskOwnerGroups( self ):
    return self.__taskManager.getTaskOwnerGroups()

  def getTask( self, taskID ):
    return self.__taskManager.getTask( taskID, 1 )

  def getTaskProgress( self, taskID ):
    return self.__taskManager.getTaskProgress( taskID )

  def getTaskInfo( self, taskID ):
    return self.__taskManager.getTaskInfo( taskID )

  def getTaskJobs( self, taskID ):
    return self.__taskManager.getTaskJobs( taskID )

  def getTaskJobsStatistics( self, taskID, statusType ):
    return self.__taskManager.getTaskJobsStatistics( taskID, statusType )

  def getTaskHistories( self, taskID ):
    return self.__taskManager.getTaskHistories( taskID )


  def getJobs( self, jobIDs, outFields ):
    return self.__taskManager.getJobs( jobIDs, outFields )


  def rescheduleTask( self, taskID, jobStatus=[] ):
    return self.__manageTask( taskID, self.__jobManager.rescheduleJob, jobStatus )


  def deleteTask( self, taskID, jobStatus=[] ):
    result = self.__manageTask( taskID, self.__jobManager.deleteJob, jobStatus )
    if not result['OK']:
      return result
    jobIDs = result['Value']['JobID']

    result = self.__taskManager.removeTask( taskID )
    if not result['OK']:
      return S_ERROR( 'Remove task error: %s' % result['Message'] )

    return S_OK( {'TaskID': taskID, 'JobID': jobIDs} )


  def __manageTask( self, taskID, action, jobStatus=[] ):
    result = self.__taskManager.getTaskJobs( taskID )
    if not result['OK']:
      return S_ERROR( 'Get task jobs error: %s' % result['Message'] )
    jobIDs = result['Value']

    if jobStatus:
      result = self.__jobMonitor.getJobs( { 'JobID': jobIDs, 'Status': jobStatus } )
      if not result['OK']:
        return S_ERROR( 'Get jobs of status %s error: %s' % (status, result['Message']) )
      jobIDs = result['Value']

    if jobIDs:
      result = action( jobIDs )
      if not result['OK']:
        return S_ERROR( 'Manage jobs error (%s): %s' % (action.__name__, result['Message']) )

    return S_OK( {'TaskID': taskID, 'JobID': jobIDs} )



""" CEBaseTest

  Base class for all the CE test classes.

"""

import os, threading
from datetime                                           import datetime, timedelta
from DIRAC                                              import S_OK
from DIRAC.Interfaces.API.Job                           import Job
from DIRAC.Interfaces.API.Dirac                         import Dirac
from DIRAC.Core.DISET.RPCClient import RPCClient
from DIRAC.ConfigurationSystem.Client.Helpers import Registry
from IHEPDIRAC.ResourceStatusSystem.SAM.SAMTest.TestBase import TestBase
from IHEPDIRAC.ResourceStatusSystem.SAM.SAMTest.TestBase import LOCK
from IHEPDIRAC.ResourceStatusSystem.Utilities import BESUtils


__RCSID__ = '$Id: $'




class CEBaseTest( TestBase ):
  """
    CEBaseTest is base class for all the CE test classes. Real  CE test should
    implement its _judge method.
  """

  def __init__( self, args = None, apis = None ):
    super( CEBaseTest, self ).__init__( args, apis )

    self.timeout = self.args.get( 'timeout', 1800 )
    self.vo = self.args.get( 'VO' )
    self.testType = self.args[ 'TestType' ]
    self.executable = self.args[ 'executable' ]
    self.__logPath = '/opt/dirac/work/ResourceStatus/SAMTestAgent/SAM/log'
    self.__scriptPath = '/opt/dirac/pro/IHEPDIRAC/ResourceStatusSystem/SAM/sam_script'

    if 'WMSAdministrator' in self.apis:
      self.wmsAdmin = self.apis[ 'WMSAdministrator' ]
    else:
      self.wmsAdmin = RPCClient( 'WorkloadManagement/WMSAdministrator' )

    if 'Dirac' in self.apis:
      self.dirac = self.apis[ 'Dirac' ]
    else:
      self.dirac = Dirac()


  def doTest( self, elementDict ):
    """
      submit test job to the specified ce or cloud..
    """

    elementName = elementDict[ 'ElementName' ]
    elementType = elementDict[ 'ElementType' ]
    vos = elementDict[ 'VO' ]

    site = None; ce = None
    if elementType == 'ComputingElement':
      ce = elementName
    if elementType == 'CLOUD':
      site = elementName

    if self.vo:
      submitVO = self.vo
    elif vos:
      submitVO = vos[ 0 ]
    else:
      submitVO = 'bes'

    submissionTime = datetime.utcnow().replace( microsecond = 0 )
    sendRes = self.__submit( site, ce, submitVO )
    if not sendRes[ 'OK' ]:
      return sendRes
    jobID = sendRes[ 'Value' ]

    result = { 'Result' : { 'JobID' : jobID,
                           'VO' : submitVO,
                           'SubmissionTime' : submissionTime },
              'Finish' : False }

    return S_OK( result )


  def __submit( self, site, CE, vo ):
    """
      set the job and submit.
    """

    job = Job()
    job.setName( self.testType )
    job.setJobGroup( 'CE-Test' )
    job.setExecutable( self.executable )
    job.setInputSandbox( '%s/%s' % ( self.__scriptPath, self.executable ) )
    if site and not CE:
      job.setDestination( site )
    if CE:
      job.setDestinationCE( CE )

    LOCK.acquire()
    proxyPath = BESUtils.getProxyByVO( 'zhangxm', vo )
    if not proxyPath[ 'OK' ]:
      LOCK.release()
      return proxyPath
    proxyPath = proxyPath[ 'Value' ]
    oldProxy = os.environ.get( 'X509_USER_PROXY' )
    os.environ[ 'X509_USER_PROXY' ] = proxyPath
    result = self.dirac.submitJob( job )
    if oldProxy is None:
      del os.environ[ 'X509_USER_PROXY' ]
    else:
      os.environ[ 'X509_USER_PROXY' ] = oldProxy
    LOCK.release()

    return result


  def getTestResult( self, elementName, vo, jobID, submissionTime ):
    """
      download output sandbox and judge the test status from the log file.
    """

    isFinish = False

    res = self.__getJobOutput( jobID, vo )
    if not res[ 'OK' ]:
      return res
    output = res[ 'Value' ]
    status = res[ 'Status' ]

    resDict = { 'CompletionTime' : None, 'Status' : None, 'Log' : None, 'ApplicationTime' : None }
    utcNow = datetime.utcnow().replace( microsecond = 0 )

    if output:
      isFinish = True
      resDict[ 'CompletionTime' ] = utcNow
      log = output[ 'Log' ]
      if not output[ 'Download' ]:
        resDict[ 'Status' ] = 'Unknown'
        resDict[ 'Log' ] = 'Fail to download log file for job %s: %s' % ( jobID, log )
      else:
        resDict[ 'Log' ] = log
        resDict[ 'Status' ] = self._judge( log )
        resDict[ 'AppliactionTime' ] = self.__getAppRunningTime( log )

    else:
      if utcNow - submissionTime >= timedelta( seconds = self.timeout ):
        isFinish = True
        if elementName.split( '.' )[ 0 ] == 'CLOUD':
          site = elementName
        else:
          site = BESUtils.getSiteForCE( elementName )
        jobCount = self.wmsAdmin.getSiteSummaryWeb( { 'Site' : site }, [], 0, 0 )
        if not jobCount[ 'OK' ]:
          return jobCount
        params = jobCount[ 'Value' ][ 'ParameterNames' ]
        records = jobCount[ 'Value' ][ 'Records' ][ 0 ]
        run = records[ params.index( 'Running' ) ]
        done = records[ params.index( 'Done' ) ]
        if status == 'Waiting' and run == 0 and done == 0:
          resDict[ 'Status' ] = 'Bad'
          resDict[ 'Log' ] = 'The test job is waiting for %d seconds, but no running and done jobs at this site.' % self.timeout
        else:
          if run != 0:
            resDict[ 'Status' ] = 'Busy'
            resDict[ 'Log' ] = 'Site %s is too busy to execute this test job, job status is %s' % ( site, status )
          else:
            resDict[ 'Status' ] = 'Unknown'
            resDict[ 'Log' ] = 'Test did not complete within the timeout of %d seconds, job status is %s' % ( self.timeout, status )
        self.dirac.killJob( jobID )

    if not isFinish:
      return S_OK()
    else:
      return S_OK( resDict )


  def __getJobOutput( self, jobID, vo ):
    status = self.dirac.getJobStatus( jobID )
    if not status[ 'OK' ]:
      return status
    status = status[ 'Value' ][ jobID ][ 'Status' ]

    if status in ( 'Done', 'Failed' ):
      LOCK.acquire()
      proxyPath = BESUtils.getProxyByVO( 'zhangxm', vo )
      if not proxyPath[ 'OK' ]:
        LOCK.release()
        return proxyPath
      proxyPath = proxyPath[ 'Value' ]
      oldProxy = os.environ.get( 'X509_USER_PROXY' )
      os.environ[ 'X509_USER_PROXY' ] = proxyPath
      outputRes = self.dirac.getOutputSandbox( jobID, self.__logPath )
      if oldProxy is None:
        del os.environ[ 'X509_USER_PROXY' ]
      else:
        os.environ[ 'X509_USER_PROXY' ] = oldProxy
      LOCK.release()

      if not outputRes[ 'OK' ]:
        ret = S_OK( { 'Download'  : False, 'Log' : outputRes[ 'Message' ] } )
      else:
        try:
          logfile = open( '%s/%d/Script1_CodeOutput.log' % ( self.__logPath, jobID ), 'r' )
          log = logfile.read()
          logfile.close()
        except IOError, e:
          raise IOError
        os.system( 'rm -rf %s/%d' % ( self.__logPath, jobID ) )
        ret = S_OK( { 'Download' : True, 'Log' : log } )
    else:
      ret = S_OK()

    ret[ 'Status' ] = status
    return ret


  @staticmethod
  def __getAppRunningTime( log ):
    index = log.find( 'Running Time :' )
    runtime = ''
    while log[ index ] != '\n':
      runtime += log[ index ]
      index += 1
    runtime = float( runtime[ len( 'Running Time :'  ) : ].strip() )

    return runtime


  @staticmethod
  def _judge( log ):
    """
      to be extended by real ce tests.
    """

    return 'OK'



# -*- coding: utf-8 -*-

__RCSID__ = "$Id: $"

import datetime

from DIRAC import gLogger, gConfig, S_OK, S_ERROR
from DIRAC.Core.DISET.RequestHandler import RequestHandler

global gTransferDB

def initializeDatasetHandler(serviceInfo):
  """ initialize handler """
  gLogger.info("Initialize Dataset Handler.")

  from IHEPDIRAC.TransferSystem.DB.TransferDB import TransferDB

  global gTransferDB

  gTransferDB = TransferDB()

  return S_OK()

class DatasetHandler(RequestHandler):
  """
  This is for:
    * create a dataset
  """

  def initialize(self):
    credDict = self.getRemoteCredentials()
    gLogger.info(credDict)
    self.user = credDict["username"]

    return S_OK()

  types_create = [str, list]
  def export_create(self, dataset, filelist):
    gLogger.info("Username: ", self.user)
    gLogger.info("Dataset: ", dataset)
    gLogger.info("Filelist: ", filelist)
    res = gTransferDB.insert_Dataset( dataset, self.user, filelist)

    if not res["OK"]:
      return res

    return S_OK()

  types_list = [ str ]
  def export_list(self, dataset):
    gLogger.info("Dataset: ", dataset)
    condDict = {'name': dataset}

    res = gTransferDB.get_Dataset(condDict)
    return res

  types_show = [ dict, [list, str], [int, long], [int, long] ]
  def export_show(self, condDict=None, orderby=None,
                        start=None, limit=None):
    """ show the datasets """
    res = gTransferDB.show_Datasets(condDict, orderby, start, limit)
    return res

  types_showtotal = [ dict ]
  def export_showtotal(self, condDict):
    res = gTransferDB.showtotal_Datasets(condDict)
    return res



# $HeadURL $
''' MySQLWrapper

  Module that provides functions needed for RSS databases and have not yet
  been incorporated to the DIRAC MySQL module.

'''

from DIRAC import S_ERROR

__RCSID__ = '$Id: $'

def _capitalize( params ):
  '''
    Capitalize first letter of all keys in dictionary
  '''

  capitalizedParams = {}
  for key, value in params.items():
    capitalizedParams[ key[0].upper() + key[1:] ] = value

  return capitalizedParams

def _discardNones( params ):
  '''
    Remove all keys with None as value in the key,value pair
  '''

  validParams = {}
  for key, value in params.items():
    if value is not None:
      validParams[ key ] = value

  return validParams

def insert( rssDB, params, meta ):
  '''
    Method that transforms the RSS DB insert into the MySQL insertFields
    method.
  '''

  # onlyUniqueKeys is not used, but if deleted here, addOrModify method crashes
  accepted_keys = ( 'table', 'onlyUniqueKeys' )

  params = _capitalize( params )
  params = _discardNones( params )

  # Protection to avoid misunderstandings between MySQLMonkey and new code.
  if set( meta.keys() ) - set( accepted_keys ):
    return S_ERROR( 'Insert statement only accepts %s, got %s' % ( accepted_keys, meta.keys() ) )

  tableName  = meta[ 'table' ]
  tablesList = rssDB.getTablesList()
  if not tablesList[ 'OK' ]:
    return tablesList

  if not tableName in tablesList[ 'Value' ]:
    return S_ERROR( '"%s" is not on the schema tables' )

  return rssDB.database.insertFields( tableName, inDict = params )

def update( rssDB, params, meta ):
  '''
    Method that transforms the RSS DB update into the MySQL updateFields
    method.
  '''

  accepted_keys = ( 'table', 'onlyUniqueKeys', 'uniqueKeys' )

  params = _capitalize( params )
  params = _discardNones( params )

  # Protection to avoid misunderstandings between MySQLMonkey and new code.
  if set( meta.keys() ) - set( accepted_keys ):
    return S_ERROR( 'Update statement only accepts %s, got %s' % ( accepted_keys, meta.keys() ) )

  tableName  = meta[ 'table' ]
  tablesList = rssDB.getTablesList()
  if not tablesList[ 'OK' ]:
    return tablesList

  if not tableName in tablesList[ 'Value' ]:
    return S_ERROR( '"%s" is not on the schema tables' % tableName )

  if 'uniqueKeys' in meta:
    uniqueKeys = meta[ 'uniqueKeys' ]
  else:
    res = rssDB.getTable( tableName )
    if not res[ 'OK' ]:
      return res
    uniqueKeys = res[ 'Value' ][ 'PrimaryKey' ]

  # Little bit messy, but we split the fields that will be used to select and
  # which ones will be updated.
  paramsToUpdate = [ ( key, value ) for ( key, value ) in params.items() if key not in uniqueKeys ]
  paramsToUpdate = dict( paramsToUpdate )

  paramsToSelect = [ ( key, value ) for ( key, value ) in params.items() if key in uniqueKeys ]
  paramsToSelect = dict( paramsToSelect )

  return rssDB.database.updateFields( tableName, condDict = paramsToSelect,
                                      updateDict = paramsToUpdate )

def select( rssDB, params, meta ):
  '''
    Method that transforms the RSS DB select into the MySQL getFields method.
  '''

  accepted_keys = ( 'table', 'columns', 'order', 'limit', 'onlyUniqueKeys', 'older', 'newer' )

  params = _capitalize( params )
  params = _discardNones( params )

  # Protection to avoid misunderstandings between MySQLMonkey and new code.
  if set( meta.keys() ) - set( accepted_keys ):
    return S_ERROR( 'Select statement only accepts %s, got %s' % ( accepted_keys, meta.keys() ) )

  tableName  = meta[ 'table' ]
  tablesList = rssDB.getTablesList()

  if not tableName in tablesList[ 'Value' ]:
    return S_ERROR( '"%s" is not on the schema tables' )

  outFields, limit, order = None, None, None
  if 'columns' in meta:
    outFields = meta[ 'columns' ]
  else:
    tableDefinition = rssDB.getTable( tableName )
    if not tableDefinition[ 'OK' ]:
      return tableDefinition
    outFields = tableDefinition[ 'Value' ][ 'Fields' ].keys()

  if 'limit' in meta:
    limit     = meta[ 'limit' ]
  if 'order' in meta:
    order     = meta[ 'order' ]

  keys = []

  if 'uniqueKeys' in meta:
    keys = meta[ 'uniqueKeys' ]
  elif 'onlyUniqueKeys' in meta:
    tableDefinition = rssDB.getTable( tableName )
    if not tableDefinition[ 'OK' ]:
      return tableDefinition
    keys = tableDefinition[ 'Value' ][ 'PrimaryKey' ]

  if keys:
    newParams = {}
    for key in keys:
      if key in params:
        newParams[ key ] = params[ key ]

    params = newParams

  field, older, newer = None, None, None
  if 'older' in meta:
    field, older = meta[ 'older' ]
  elif 'newer' in meta:
    field, newer = meta[ 'newer' ]

  selectResult = rssDB.database.getFields( tableName, condDict = params,
                                           outFields = outFields, limit = limit,
                                           orderAttribute = order, older = older,
                                           newer = newer, timeStamp = field )
  selectResult[ 'Columns' ] = outFields
  return selectResult

def delete( rssDB, params, meta ):
  '''
    Method that transforms the RSS DB delete into the MySQL
  '''

  accepted_keys = ( 'table', 'older', 'newer' )

  params = _capitalize( params )
  params = _discardNones( params )

  # Protection to avoid misunderstandings between MySQLMonkey and new code.
  if set( meta.keys() ) - set( accepted_keys ):
    return S_ERROR( 'Delete statement only accepts %s, got %s' % ( accepted_keys, meta.keys() ) )

  tableName  = meta[ 'table' ]
  tablesList = rssDB.getTablesList()
  if not tablesList[ 'OK' ]:
    return tablesList

  if not tableName in tablesList[ 'Value' ]:
    return S_ERROR( '"%s" is not on the schema tables' )

  field, older, newer = None, None, None
  if 'older' in meta:
    field, older = meta[ 'older' ]
  elif 'newer' in meta:
    field, newer = meta[ 'newer' ]

  # Small secutiry measure, this prevents full table deletion.. by mistake I hope.
  if not params and not field and not ( older or newer ):
    return S_ERROR( 'Dude, you are going to delete the whole table %s' % tableName )

  return rssDB.database.deleteEntries( tableName, condDict = params, older = older,
                                       newer = newer, timeStamp = field )

################################################################################
#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF



''' CLOUDAccessTest

A test class to test the access to clouds.

'''

from urlparse import urlparse

from DIRAC import S_OK, S_ERROR, gConfig


class CLOUDAccessTest:
  ''' CLOUDAccessTest
  '''

  def _getAccessParams( self, element ):
    '''
      get the access host and port for the specified cloud.
    '''

    _basePath = 'Resources/Sites'
    _searchKey = ( 'ex_force_auth_url', 'EndpointUrl' )

    cloud = gConfig.getSections( '%s/CLOUD/%s/Cloud' % ( _basePath, element ) )
    if not cloud[ 'OK' ]:
      return cloud
    cloud = cloud[ 'Value' ][ 0 ]

    for key in _searchKey:
      url = gConfig.getValue( '%s/CLOUD/%s/Cloud/%s/%s' % ( _basePath, element, cloud, key ) )
      if url:
        o = urlparse(url)
        if o.port:
          port = o.port
        else:
          port = 443 if o.scheme == 'https' else 80
        return S_OK((o.hostname, port))

    return S_ERROR('%s is not a vaild CLOUD.' % element)



import os
import urllib
import urllib2
import shutil

from DIRAC import gConfig, gLogger

from IHEPDIRAC.Badger.private.output.getfile.GetFile import GetFile

class HttpGetFile(GetFile):
  def __init__(self):
    super(HttpGetFile, self).__init__()

    self.__url = gConfig.getValue('/Resources/Applications/DataLocation/Http/Url', 'http://bes-srm.ihep.ac.cn:2880/bes')
    gLogger.debug('HTTP url:', self.__url)

  def _lfnToRemote(self, lfn):
    return self.__url + lfn

  def _available(self):
    try:
      req = urllib2.urlopen(urllib.quote(self.__url, '/:'))
    except Exception, e:
      return False
    return True

  def _downloadSingleFile(self, remotePath, localPath):
    gLogger.debug('getfile from %s to %s' % (remotePath, localPath))
    try:
      req = urllib2.urlopen(urllib.quote(remotePath, '/:'))
    except Exception, e:
      gLogger.debug('HTTP get file error:', e)
      return False

    with open(localPath, 'wb') as fp:
      shutil.copyfileobj(req, fp)
    return True



''' SAMTestAgent

  This agent executes SAM tests and evaluates resources and sites SAM status.

'''

import threading
from datetime import datetime
from DIRAC                                             import S_OK, gConfig, gLogger
from DIRAC.Core.Base.AgentModule                       import AgentModule
from DIRAC.Core.DISET.RPCClient                        import RPCClient
from IHEPDIRAC.ResourceStatusSystem.Utilities          import CSHelpers
from DIRAC.DataManagementSystem.Client.DataManager                 import DataManager
from DIRAC.ResourceStatusSystem.Utilities import Utils
from IHEPDIRAC.ResourceStatusSystem.Client.ResourceManagementIHEPClient import ResourceManagementIHEPClient
from IHEPDIRAC.ResourceStatusSystem.SAM.TestExecutor                import TestExecutor
from IHEPDIRAC.ResourceStatusSystem.SAM.StatusEvaluator             import StatusEvaluator
from IHEPDIRAC.ResourceStatusSystem.Utilities import BESUtils
from DIRAC.Interfaces.API.DiracAdmin import DiracAdmin


__RCSID__ = '$Id:  $'
AGENT_NAME = 'ResourceStatus/SAMTestAgent'


class SAMTestAgent(AgentModule):
  """ SAMTestAgent

    The SAMTestAgent is used to execute SAM tests and evaluate SAM status
    periodically. It executes tests with TestExecutor and evaluates status with
    StatusEvaluator.
  """

  def __init__(self, *args, **kwargs):
    AgentModule.__init__(self, *args, **kwargs)

    self.tests = {}
    self.apis = {}


  def initialize(self):
    """
      specify the tests which need to be executed.
    """


    self.apis[ 'DataManager' ] = DataManager()
    self.apis[ 'ResourceManagementIHEPClient' ] = ResourceManagementIHEPClient()

    return S_OK()


  def execute(self):
    """
      The main method of the agent. It get elements which need to be tested and
      evaluated from CS. Then it instantiates TestExecutor and StatusEvaluate and
      calls their main method to finish all the work.
    """

    from IHEPDIRAC.ResourceStatusSystem.SAM.SAMTest import TestConfiguration
    self.tests = TestConfiguration.TESTS
    self.__loadTestObj()

    self.testExecutor = TestExecutor( self.tests, self.apis )
    self.statusEvaluator = StatusEvaluator( self.apis )

    elements = []
    sitesCEs = {}

    # CE tests
    noTestSites = [ site.strip() for site in self.am_getOption( 'noTestSite', '' ).split( ',' ) if site != '' ]
    diracAdmin = DiracAdmin()
    activeSites = diracAdmin.getSiteMask()
#    wmsAdmin = RPCClient('WorkloadManagement/WMSAdministrator')
#    activeSites = wmsAdmin.getSiteMask()
    if not activeSites[ 'OK' ]:
      return activeSites
    activeSites = [ site for site in activeSites[ 'Value' ] if site not in noTestSites ]
    gLogger.info('Active sites: %s', activeSites)

    for siteName in activeSites:
      domain = siteName.split('.')[ 0 ]
      vos = BESUtils.getSiteVO( siteName )
      if 'CLOUD' != domain:
        siteCEs = CSHelpers.getSiteComputingElements( siteName )
        sitesCEs[ siteName ] = siteCEs
        for ce in siteCEs:
          elements.append( { 'ElementName' : ce,
                                                  'ElementType' : 'ComputingElement',
                                                  'VO' : vos } )
          gLogger.debug("List of elements: %s" % ce)

      else:
        sitesCEs[ siteName ] = [ siteName ]
        elements.append( { 'ElementName' : siteName,
                                                'ElementType' : 'CLOUD',
                                                'VO' : vos } )

    # SE tests
    ses = gConfig.getValue( 'Resources/StorageElementGroups/SE-USER' )
    for se in ses.split( ', ' ):
      seSites = BESUtils.getSitesForSE( se )
      for seSite in seSites:
        gLogger.debug( 'Site for SE %s: %s' % (se, seSite) )
        if seSite not in activeSites:
          continue
        vos = BESUtils.getSiteVO( seSite )
        gLogger.debug( 'vos for SE %s under site %s: %s' % (se, seSite, vos) )
        if len(vos) == 0:
          continue
        vo = vos[0]
        elements.append( { 'ElementName' : se,
                                              'ElementType' : 'StorageElement',
                                              'VO' : vo } )
        gLogger.info( 'VO for SE %s: %s' % ( se, vo ) )
        break

    lastCheckTime = datetime.utcnow().replace(microsecond = 0)
    self.elementsStatus = {}

    threads = []
    for elementDict in elements:
      gLogger.info( 'Start SAM test for %s with VO %s' % ( elementDict['ElementName'], elementDict.get('VO', []) ) )
      t = threading.Thread( target = self._execute, args = ( elementDict, ) )
      threads.append( t )
      t.start()

    for thread in threads:
      thread.join()

    for siteName in activeSites:
      seList = CSHelpers.getSiteStorageElements( siteName )
      se = ''
      if [] != seList:
        se = seList[ 0 ]
      try:
        seStatus = self.elementsStatus[ se ][ 'all' ]
      except KeyError:
        seStatus = None

      voStatus = { 'all' : [] }
      for ce in sitesCEs[ siteName ]:
        if not self.elementsStatus.has_key( ce ):
          continue
        for vo, status in self.elementsStatus[ ce ].items():
          if vo not in voStatus:
            voStatus[ vo ] = []
          voStatus[ vo ].append( status )

      for vo, ceStatusList in voStatus.items():
        if ceStatusList == [] and seStatus == None:
          continue
        res = self.statusEvaluator.evaluateSiteStatus( siteName, ceStatusList, seStatus, vo = vo, lastCheckTime = lastCheckTime)
        if not res[ 'OK' ]:
          gLogger.error( 'StatusEvaluator.evaluateSiteStatus: %s' % res[ 'Message' ] )
          break

    return S_OK()


  def _execute( self, elementDict ):
    elementName = elementDict[ 'ElementName' ]
    vos = elementDict.get( 'VO', [] )
    utcNow = datetime.utcnow().replace( microsecond = 0 )

    # Return status of each Testtype
    testRes = self.testExecutor.execute( elementDict, lastCheckTime = utcNow )
    if not testRes[ 'OK' ]:
      gLogger.error( 'TestExecutor.execute error for %s: %s' % ( elementName, testRes[ 'Message' ] ) )
      return
    testsStatus = testRes[ 'Value' ]
    gLogger.info( 'The status of %s is %s' % ( elementName, testsStatus ) )

    defaultTestsStatus = {}
    voTestsStatus = {}
    for vo in vos:
      voTestsStatus[ vo ] = {}
    for testType, status in testsStatus.items():
      vo = self.tests[ testType ][ 'match' ].get( 'VO' )
      if not vo:
        defaultTestsStatus[ testType ] = status
      else:
        voTestsStatus[ vo ][ testType ] = status

    elementStatus = {}
    self.elementsStatus[ elementName ] = elementStatus

    res = self.statusEvaluator.evaluateResourceStatus( elementDict, testsStatus, lastCheckTime = utcNow )
    if not res[ 'OK' ]:
      gLogger.error( 'StatusEvaluator.evaluateResourceStatus: %s' % res[ 'Message' ] )
      return
    elementStatus[ 'all' ] = res[ 'Value' ]

    for vo, statuses in voTestsStatus.items():
      statuses.update( defaultTestsStatus )
      res = self.statusEvaluator.evaluateResourceStatus( elementDict, statuses, vo = vo, lastCheckTime = utcNow )
      if not res[ 'OK' ]:
        gLogger.error( 'StatusEvaluator.evaluateResourceStatus: %s' % res[ 'Message' ] )
        return
      elementStatus[ vo ] = res[ 'Value' ]


  def __loadTestObj(self):
    _module_pre = 'DIRAC.ResourceStatusSystem.SAM.SAMTest.'

    for testType, testDict in self.tests.items():
      moduleName = testDict[ 'module' ]
      args = testDict.get( 'args', {} )
      args.update( testDict[ 'match' ] )
      args[ 'TestType' ] = testType
      try:
        testModule = Utils.voimport( _module_pre + moduleName )
      except ImportError, e:
        gLogger.error( "Unable to import %s, %s" % ( _module_pre + moduleName, e ) )
        continue
      testClass = getattr( testModule, moduleName )
      obj = testClass(args, self.apis)
      testDict[ 'object' ] = obj



# -*- coding: utf-8 -*-

from DIRAC import gLogger, gConfig, S_OK, S_ERROR
from DIRAC.Core.Base.DB import DB
from DIRAC.Core.Utilities.MySQL import _quotedList

# Some basic arguments will use namedtuple 
from collections import namedtuple
import datetime

TransRequestEntry = namedtuple('TransRequestEntry',
                              [#'id',
                               'username',
                               'dataset',
                               'srcSE',
                               'dstSE',
                               'protocol',
                               'submit_time',
                               'status',
                               ])
TransRequestEntryWithID = namedtuple('TransRequestEntryWithID',
                                      ('id',) + TransRequestEntry._fields)
TransFileListEntry = namedtuple('TransFileListEntry',
                                [#'id',
                                 'LFN',
                                 'trans_req_id',
                                 'start_time',
                                 'finish_time',
                                 'status',
                                 'error',
                                 ])
TransFileListEntryWithID = namedtuple('TransFileListEntryWithID',
                                      ('id',) + TransFileListEntry._fields)

DatasetEntry = namedtuple('DatasetEntry',
                          [#'id',
                            'name',
                            'username',
                            ])
DatasetEntryWithID = namedtuple('DatasetEntryWithID',
                                ('id',) + DatasetEntry._fields)
FilesInDatasetEntry = namedtuple('FilesInDatasetEntry',
                                [#'id',
                                  'LFN',
                                  'dataset_id',
                                  ])
FilesInDatasetEntryWithID = namedtuple('FilesInDatasetEntryWithID',
                                ('id',) + FilesInDatasetEntry._fields)


class TransferDB(DB):
  tables = dict(TransferRequest = "TransferRequest",
                TransferFileList = "TransferFileList",
                Dataset = "Dataset",
                FilesInDataSet = "FilesInDataSet")

  def __init__(self, dbname="TransferDB", 
                     fullname="Transfer/TransferDB",
                     maxQueueSize = 10):
    DB.__init__(self, dbname, fullname, maxQueueSize)


  """override the original getFields"""
  def getFields( self, tableName, outFields = None,
                 condDict = None, offset = None,
                 limit = False, conn = None,
                 older = None, newer = None,
                 timeStamp = None, orderAttribute = None,
                 greater = None, smaller = None ):
    """
      Select "outFields" from "tableName" with condDict
      N records can match the condition
      return S_OK( tuple(Field,Value) )
      if outFields == None all fields in "tableName" are returned
      if limit is not False, the given limit is set
      inValues are properly escaped using the _escape_string method, they can be single values or lists of values.
    """
    table = _quotedList( [tableName] )
    if not table:
      error = 'Invalid tableName argument'
      self.log.warn( 'getFields:', error )
      return S_ERROR( error )

    quotedOutFields = '*'
    if outFields:
      quotedOutFields = _quotedList( outFields )
      if quotedOutFields == None:
        error = 'Invalid outFields arguments'
        self.log.warn( 'getFields:', error )
        return S_ERROR( error )

    self.log.verbose( 'getFields:', 'selecting fields %s from table %s.' %
                          ( quotedOutFields, table ) )

    if condDict == None:
      condDict = {}

    try:
      condition = self.buildCondition( condDict = condDict, older = older, newer = newer,
                        timeStamp = timeStamp, orderAttribute = orderAttribute, limit = limit,
                        offset = offset,
                        greater = None, smaller = None )
    except Exception, x:
      return S_ERROR( x )

    return self._query( 'SELECT %s FROM %s %s' %
                        ( quotedOutFields, table, condition ), conn, debug = True )

  def insert_TransferRequest(self, entry):
    if not isinstance(entry, TransRequestEntry):
      raise TypeError("entry should be TransRequestEntry")
    infoDict = dict(entry._asdict())
    res = self.insertFields( self.tables["TransferRequest"],
                             inDict = infoDict)
    if not res["OK"]:
      return res
    res = self._query("select last_insert_id()")
    if not res["OK"]:
      return res
    id = res['Value'][0][0]
    return S_OK(id)

  def get_TransferRequest(self, condDict = None):
    res = self.getFields( self.tables["TransferRequest"],
                          outFields = TransRequestEntryWithID._fields,
                          condDict = condDict,
                          )
    return res


  def get_TransferRequestTotal(self, condDict = None):
    res_total = self.countEntries(self.tables["TransferRequest"],
                                  condDict=condDict)
    return res_total

  def get_TransferRequestWithLimit(self, condDict=None, orderby=None, offset=None, limit=None):
    """
    >>> orderby = "id"
    or
    >>> orderby = ["id:DESC", "name:ASC"]

    >>> offset = 5 # begin
    >>> limit = 5 # total entries
    """
    res = self.getFields( self.tables["TransferRequest"],
                          outFields = TransRequestEntryWithID._fields,
                          condDict = condDict,
                          orderAttribute = orderby,
                          limit = limit,
                          offset = offset,
                          )
    return res

  def insert_PerTransferFile(self, entry):
    if not isinstance(entry, TransFileListEntry):
      raise TypeError("entry should be TransFileListEntry")
    infoDict = dict(entry._asdict())
    res = self.insertFields( self.tables["TransferFileList"],
                             inDict = infoDict)
    if not res["OK"]:
      return res
    res = self._query("select last_insert_id()")
    return res

  def insert_TransferFileList(self, trans_req_id, filelist):
    # << get list of files for the dataset >>
    for dsfile in map(FilesInDatasetEntryWithID._make, filelist):
      entry = TransFileListEntry(LFN = dsfile.LFN,
                                 trans_req_id = trans_req_id,
                                 start_time = '0000-00-00',
                                 finish_time = '0000-00-00',
                                 status = "new",
                                 error = "",
                                 )
      self.insert_PerTransferFile(entry)

  def get_TransferFileList(self, condDict = None):
    res = self.getFields( self.tables["TransferFileList"], 
                           outFields = TransFileListEntryWithID._fields,
                           condDict = condDict,
                           )
    return res

  def delete_TransferFileListByReq(self, condDict = None):
    """
    condDict = {"trans_req_id": int(transid)}
    """
    # get the file list in req.
    res = self.getFields( self.tables["TransferFileList"], 
                           outFields = TransFileListEntryWithID._fields,
                           condDict = condDict,
                           )
    if not res["OK"]:
      gLogger.error(res)
      return res

    if not res["Value"]:
      gLogger.error(res)
      return res

    # remove these.

    for rawentry in res["Value"]:
      entry = TransFileListEntryWithID._make(rawentry)
      self.delete_TransferFileList( {"id":entry.id})
    return S_OK()

  def delete_TransferFileList(self, condDict = None):
    """ currently, condDict should be
        {
          "id": 154
        }
    """
    if condDict is None:
      return S_ERROR("No File ID")
    if not condDict.has_key("id"):
      return S_ERROR("The dict need a key called 'id'")
    if not str(condDict["id"]).isdigit():
      return S_ERROR("The type of dict['id'] should be int")
    fileid = int(str(condDict["id"]))

    # check fileid status
    # if it is finish, we don't kill it any more
    res = self.getFields( self.tables["TransferFileList"], 
                          outFields=["status"], 
                          condDict={"id":fileid})
    #print res
    if not res["OK"]:
      gLogger.error(res)
      return res

    if not res["Value"]:
      return res

    # TODO XXX
    if len(res["Value"]) != 1:
      gLogger.error(res)
      return res

    # TODO
    status = res["Value"][0][0]
    if status == "finish":
      return S_ERROR("It is finished. Can't Kill.")
    toUpdate = {"status": "kill"}
    res = self.updateFields(
                              self.tables["TransferFileList"],
                              updateDict = toUpdate,
                              condDict = {"id":fileid},
                              )
    return res

  def retransfer_TransferFileList(self, condDict = None):
    """ currently, condDict should be
        {
          "id": 154
        }
    """
    if condDict is None:
      return S_ERROR("No File ID")
    if not condDict.has_key("id"):
      return S_ERROR("The dict need a key called 'id'")
    if not str(condDict["id"]).isdigit():
      return S_ERROR("The type of dict['id'] should be int")
    fileid = int(str(condDict["id"]))

    # check status is finish or kill
    res = self.getFields( self.tables["TransferFileList"], 
                          outFields=TransFileListEntry._fields, 
                          condDict={"id":fileid})
    #print res
    if not res["OK"]:
      gLogger.error(res)
      return res

    if not res["Value"]:
      return res

    # TODO XXX
    if len(res["Value"]) != 1:
      gLogger.error(res)
      return res

    # TODO
    entry = TransFileListEntry._make(res["Value"][0])
    status = entry.status
    if status not in ("finish", "kill"):
      return S_ERROR("The status should be kill or finish")

    # TODO
    # create a new Transfer File
    entry = entry._replace(status = "new", error="")
    res = self.insert_PerTransferFile(entry)
    if not res["OK"]:
      gLogger.error(res)
      return res

    # TODO
    # update the status of the transfer request.
    res = self.updateFields(
                self.tables["TransferRequest"],
                updateDict = {"status": "transfer"},
                condDict = {"id": entry.trans_req_id}
                )
    return res

  def insert_Dataset(self, dataset, user, filelist):
    # two step:
    # insert into Dataset table.
    # insert into Files in Dataset table.
    entry = DatasetEntry( name = dataset,
                          username = user,
                          )
    res = self.helper_insert_Dataset_table(entry)
    if not res["OK"]:
      return res
    dataset_id = res["Value"]

    for perfile in filelist:
      entry = FilesInDatasetEntry( dataset_id = dataset_id,
                                   LFN = perfile)
      res = self.helper_insert_FilesInDataset_table(entry)
      if not res["OK"]:
        return res
    return S_OK()

  def get_Dataset(self, condDict = None):
    res = self.get_DatasetInfo(condDict)
    if not res["OK"]:
      return res
    filelist = []
    for entry in map(DatasetEntryWithID._make, res["Value"]):
      res2 = self.get_FilesInDataset( {"dataset_id": entry.id } )
      if not res2:
        return res2
      filelist . extend ( res2["Value"] )
    print filelist
    return S_OK(filelist)

  def get_DatasetInfo(self, condDict = None):
    res = self.getFields( self.tables["Dataset"], 
                          outFields = DatasetEntryWithID._fields,
                          condDict = condDict,
                        )
    return res

  def get_FilesInDataset(self, condDict = None):
    res = self.getFields( self.tables["FilesInDataSet"],
                          outFields = FilesInDatasetEntryWithID._fields,
                          condDict = condDict,
                        )
    return res

  def show_Datasets(self, condDict=None, orderby =None,
                          start=None, limit=None):
    res = self.getFields( self.tables["Dataset"],
                          outFields = DatasetEntryWithID._fields,
                          condDict = condDict,
                          orderAttribute = orderby,
                          limit = limit,
                          offset = start)
    return res

  def showtotal_Datasets(self, condDict=None):
    res_total = self.countEntries(self.tables["Dataset"],
                                  condDict=condDict)
    return res_total

  def helper_insert_Dataset_table(self, entry):
    if not isinstance(entry, DatasetEntry):
      raise TypeError("entry should be DatasetEntry")
    infoDict = dict(entry._asdict())
    res = self.insertFields( self.tables["Dataset"],
                             inDict=infoDict,
                             )
    if not res["OK"]:
      return res
    res = self._query("select last_insert_id()")
    if not res["OK"]:
      return res
    id = res['Value'][0][0]
    return S_OK(id)

  def helper_insert_FilesInDataset_table(self, entry):
    if not isinstance(entry, FilesInDatasetEntry):
      raise TypeError("entry should be FilesInDatasetEntry")
    infoDict = dict(entry._asdict())
    res = self.insertFields( self.tables["FilesInDataSet"],
                             inDict=infoDict,
                             )
    return res

if __name__ == "__main__":
  from DIRAC.Core.Base import Script
  Script.parseCommandLine( ignoreErrors = True )

  gDB = TransferDB()

  entry = TransRequestEntry(username = "lintao", 
                            dataset = "my-dataset",
                            srcSE = "IHEP-USER",
                            dstSE = "IHEPD-USER",
                            protocol = "DIRACDMS",
                            status = "new",
                            submit_time = datetime.datetime.utcnow())
  res = gDB.insert_TransferRequest(entry)
  trans_id = 1
  if res["OK"]:
    trans_id = res["Value"]
    print trans_id
  print gDB.get_TransferRequest()
  print gDB.get_TransferRequest(condDict = {"id":1})

  entry = TransFileListEntry(LFN = "/path/does/not/exist",
                             trans_req_id = trans_id,
                             start_time = None,
                             finish_time = None,
                             status = "new",
                             error = "",
                             )
  gDB.insert_PerTransferFile(entry)

  filelist = map(str, range(10))

  condDict = {'trans_req_id': trans_id}
  print gDB.get_TransferFileList(condDict)
  condDict = {'status': 'new'}
  print gDB.get_TransferFileList(condDict)

  gDB.insert_Dataset( "my-dataset", "lintao", filelist)

  condDict = {'name': 'my-dataset-2'}
  print gDB.get_DatasetInfo(condDict)
  condDict = {}
  res = gDB.get_Dataset(condDict)
  if res["OK"]:
    filelist = res["Value"]
    gDB.insert_TransferFileList(trans_id, filelist)



from DIRAC                                              import S_OK
from DIRAC.ResourceStatusSystem.PolicySystem.PolicyBase import PolicyBase

__RCSID__ = '$Id:  $'

class SiteSAMPolicy( PolicyBase ):

  @staticmethod
  def _evaluate( commandResult ):
    result = {
              'Status' : None,
              'Reason' : None
              }

    if not commandResult[ 'OK' ]:
      result[ 'Status' ] = 'Error'
      result[ 'Reason' ] = commandResult[ 'Message' ]
      return S_OK( result )

    commandResult = commandResult[ 'Value' ]

    if not commandResult:
      result[ 'Status' ] = 'Unknown'
      result[ 'Reason' ] = 'No sam result to take a decision'
      return S_OK( result )

    status = {}
    for res in commandResult:
      status[ res[ 'VO' ] ] = { 'Status' : res[ 'Status' ], 'CEStatus' : res[ 'CEStatus' ], 'SEStatus' : res[ 'SEStatus' ] }

    if status[ 'all' ][ 'Status' ] == 'OK' or status[ 'all' ][ 'Status' ] == 'Busy':
      result[ 'Status' ] = 'Active'
      result[ 'Reason' ] = 'Site SAM status is %s' % status[ 'all' ][ 'Status' ]
    elif status[ 'all' ][ 'Status' ]  == 'Unknown':
      result[ 'Status' ] = 'Degraded'
      proVOs = []
      for vo, statusDict in status.items():
        if vo != 'all' and statusDict[ 'CEStatus' ] == 'Unknown':
          proVOs.append( vo )
      proVOs = ', '.join( proVOs )
      result[ 'Reason' ] = 'CEs could have some problems running the %s jobs' % proVOs
    else:
      if status[ 'all' ][ 'CEStatus' ] == 'Bad':
        fineVOsCount = 0
        proVOs = []
        for vo, statusDict in status.items():
          if vo != 'all':
            if statusDict[ 'CEStatus' ] == 'Bad':
              proVOs.append( vo )
            else:
              fineVOsCount += 1
        if fineVOsCount == 0:
          result[ 'Status' ] = 'Banned'
          result[ 'Reason' ] = 'CEs have some problems'
        else:
          result[ 'Status' ] = 'Degraded'
          result[ 'Reason' ] = 'CEs can not run the %s jobs properly' % proVOs
        proVOs = ', '.join( proVOs )
      else:
        result[ 'Status' ] = 'Degraded'
        result[ 'Reason' ] = 'SE has some problems'

    return S_OK( result )



# -*- coding: utf-8 -*-
from IHEPDIRAC.TransferSystem.Agent.helper.TransferFactory.ITransferWorker import *

class DIRACFTSTransferWorker(ITransferWorker):

  def build_cmd(self, info):
    LFN = info["LFN"]
    srcSE = info["srcSE"]
    dstSE = info["dstSE"]

    cmd_list = ["dirac-dms-fts-submit", 
                  LFN,
                  srcSE,
                  dstSE]
    return cmd_list

  def handle_exit(self, returncode):
    return  self._buffer 
    if returncode is None:
      return
    if returncode != 0:
      return  self._buffer 

  def handle_line(self, line):
    return line



if __name__ == "__main__":

  import DIRAC
  from DIRAC.Core.Base import Script
  Script.parseCommandLine( ignoreErrors = True )

  dtw = DIRACFTSTransferWorker()
  info = {"LFN":"/bes/user/z/zhangxm/dataTest/file9",
          "srcSE": "IHEP-USER",
          "dstSE": "JINR-USER"}
  dtw.create_popen(info)

  returncode = dtw.get_retcode()
  while returncode is None:
    time.sleep(0.2)
    dtw.handle_waiting()
    returncode = dtw.get_retcode()
  else:
    dtw.handle_waiting()
    print dtw.handle_exit(returncode)

  print "Work Done."



#!/usr/bin/env python

import os,sys,time
from DIRAC.Core.Base import Script
Script.initialize()

from DIRAC.Core.Utilities.ReturnValues import returnSingleResult

from DIRAC.DataManagementSystem.Client.FileCatalogClientCLI import FileCatalogClientCLI
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
from DIRAC.Core.Security.ProxyInfo import getProxyInfo

from DIRAC.Interfaces.API.Dirac import Dirac
from DIRAC import gLogger,S_OK,S_ERROR

from IHEPDIRAC.Badger.DataLoader.DFC.readAttributes import DataAll,Others
from IHEPDIRAC.Badger.DataLoader.DFC.judgeType import judgeType
"""This is the public API for BADGER, the BESIII Advanced Data ManaGER.

   BADGER wraps the DIRAC File Catalog and related DIRAC methods for 
   use in the BESIII distributed computing environment.


"""

class Badger:

    def __init__(self, fcClient = False):
        """Internal initialization of Badger API.
        """       
        if not fcClient:
            _fcType = 'DataManagement/FileCatalog'
            self.client = FileCatalogClient(_fcType)
        else:
            self.client = fcClient
        #self.besclient = FileCatalogClient('DataManagement/DatasetFileCatalog')
    def __createQuery(metaSelections):
        """
        need the function then,need not import
        from DIRAC.DataManagementSystem.Client.FileCatalogClientCLI import FileCatalogClientCLI
        TODO...
        """
        pass

    def getDatasetNamePrefix(self):
        """descide the prefix of a datasetName"""
        prefix = ''
        result = getProxyInfo(False,False)
        if result['OK']:
          userGroup = result['Value']['group']
          if userGroup=='bes_user':
            prefix = 'User_'
          elif userGroup=='production':
            prefix = 'Proc_'
          return prefix
        else:
          return prefix

        
    def getFilenamesByLocaldir(self,localDir):
        """ get all files under the given dir
        example:getFilenamesByLocaldir("/bes3fs/offline/data/663-1/4260/dst/121215/")
        result = [/bes3fs/offline/data/663-1/4260/dst/121215/filename1,
                  /bes3fs/offline/data/663-1/4260/dst/121215/filename2,
                  ...
                  ] 
        """
        fileList = []
        for rootdir,subdirs,files in os.walk(localDir):
          for name in files:
            fullPath = os.path.join(rootdir,name)
            fileList.append(fullPath)
        fileList.sort()
        return fileList

    def __getFileAttributes(self,fullPath):
        """ get all attributes of the given file,return a attribute dict.
        """
        if os.path.exists(fullPath):
          type = judgeType(fullPath)
          if type=="all":
            obj = DataAll(fullPath)
          elif type=="others":
            obj = Others(fullPath)
          elif type==None:
            errorMes= "name if %s is not correct"%fullPath
            print "cannot get attributes of %s"%fullPath
            attributes = {}
            return attributes
            #raise TypeError(errorMes)
          attributes = obj.getAttributes()
          #attributes['date'] = time.strftime('%Y-%m-%d %H:%M:%S',time.localtime())
        else:
          attributes = {}

        return attributes

    def testFunction(self):
        result = self.__getFileAttributes('/besfs2/offline/data/664-1/jpsi/dst/090613/run_0009952_All_file019_SFO-2.dst')
        print result

    def __registerDir(self,dir):
        """Internal function to register a new directory in DFC .
           Returns True for success, False for failure.
        """
        fc = self.client
        result = fc.createDirectory(dir)
        if result['OK']:
            if result['Value']['Successful']:
                if result['Value']['Successful'].has_key(dir):
                    return S_OK() 
                elif result['Value']['Failed']:
                    if result['Value']['Failed'].has_key(dir):
                        print 'Failed to create directory %s:%s'%(dir,result['Value']['Failed'][dir])
                        return S_ERROR(result) 
            else:
              return S_ERROR(result)
        else:
            print 'Failed to create directory %s:%s'%(dir,result['Message'])
            return S_ERROR(result) 
    def __registerFileMetadata(self,lfn,attributes):
        """Internal function to set metadata values on a given lfn. 
          Returns True for success, False for failure.
        """
        metadataDict = {}
        metadataDict['runL'] = attributes['runL']
        metadataDict['runH'] = attributes['runH']
        metadataDict['status'] = attributes['status']
        metadataDict['eventNumber'] = attributes['eventNumber']
        metadataDict['count'] = attributes['count']
        result = self.client.setMetadata(lfn,metadataDict)
        if not result['OK']:
          return S_ERROR() 
        else:
          return S_OK() 

    def __registerDirMetadata(self,dir,metaDict):
        """Internal function to set metadata to a directory
           Returns True for success, False for failure.
        """
        fc = self.client
        result = fc.setMetadata(dir,metaDict)
        if result['OK']:
            return S_OK() 
        else:
            message = "Error for setting metadata %s to %s: %s"%(metaDict,dir,result['Message'])
            return S_ERROR('Message') 
        
    def __dirExists(self,dir,parentDir):
        """ Internal function to check whether 'dir' is the subdirectory of 'parentDir'
            Returns 1 for Yes, 0 for NO
        """
        fc = self.client
        dir_exists = 0
        result = fc.listDirectory(parentDir)
        if result['OK']:
            for i,v in enumerate(result['Value']['Successful'][parentDir]['SubDirs']):
                if v == dir: 
                    dir_exists = 1
                    break
        else:
            print 'Failed to list subdirectories of %s:%s'%(parentDir,result['Message'])
        
        return dir_exists

    def __registerSubDirs(self,dirs_dict,dirs_meta):
        """Internal function to create directories in dirs_dict
           Returns True for sucess, False for failure
        """
        creation_ok = True
        
        for dir in dirs_dict:
            if (dir != 'dir_file')&(dir !='dir_data_mc' ):
                if self.__registerDir(dirs_meta[dir][0])['OK']:
                    result = self.__registerDirMetadata(dirs_meta[dir][0],{dir.split('_')[1]:dirs_meta[dir][1]})
                    if not result['OK']:
                        creation_ok = False
                        break
                else:
                    print 'Failed to create %s'%dir
                    creation_ok = False
                    break
            else:
                if not self.__registerDir(dirs_meta[dir])['OK']:
                    print 'Failed to create %s'%dir
                    creation_ok = False
                    break

        return creation_ok
            
    def registerHierarchicalDir(self,metaDict,rootDir='/bes'):
        """
           Create a hierarchical directory according the metadata dictionary
           Return created directory  for sucess,if this directory has been created, return this existing directory .

           Structure of the hierarchical directory:
           for real data:/bes/File/resonance/boss version/data/eventType/round
           for mc data:/bes/File/resonance/boss version/mc/eventType/round/streamId
           The eventType of all real datas is all. 

           Example:
           >>>metaDict = {'dataType': 'dst', 'eventType': 'all', 'streamId': 'stream0','resonance': 'psipp', 'round':'round01','bossVer': '6.6.1'}
           
           >>>badger.registerHierarchicalDir(metaDic)
           1
        """
        #Save about 20 lines compared with last one
        fc = self.client
        
        dir_exists = 0
        #0 for failure,1 for success,2 for existing directory
        creation_OK = 0
        lastDirMetaDict = {'dataType':metaDict['dataType'],'streamId':metaDict['streamId']}

        dir_file = rootDir + '/File'
        dir_resonance = dir_file + '/' + metaDict['resonance']
        dir_bossVer = dir_resonance + '/' + metaDict['bossVer']

        if metaDict['streamId'] == 'stream0':
            dir_data_mc = dir_bossVer + '/data'
        else:
            dir_data_mc = dir_bossVer + '/mc'
        dir_eventType = dir_data_mc + '/' +metaDict['eventType']
        dir_round = dir_eventType + '/' + metaDict['round']
        dir_streamId = dir_round + '/' + metaDict['streamId']

        # if dir_round has been created,create_round=1 
        create_round = 0

        dirs_dict = ['dir_file','dir_resonance','dir_bossVer','dir_data_mc','dir_eventType','dir_round']
        dirs_meta = {'dir_file':dir_file,'dir_data_mc':dir_data_mc,'dir_resonance':[dir_resonance,metaDict['resonance']],'dir_bossVer':[dir_bossVer,metaDict['bossVer']],'dir_eventType':[dir_eventType,metaDict['eventType']],'dir_round':[dir_round,metaDict['round']]}
        dir_exists = self.__dirExists(dir_file,rootDir)
        if not dir_exists:
            result = self.__registerSubDirs(dirs_dict,dirs_meta)
            if result:
                create_round = 1
        else:
            dir_exists = self.__dirExists(dir_resonance,dir_file)
            if not dir_exists:
                dirs_dict = dirs_dict[1:]
                result = self.__registerSubDirs(dirs_dict,dirs_meta)
                if result:
                    create_round = 1
            else:
                dir_exists = self.__dirExists(dir_bossVer,dir_resonance)
                if not dir_exists:
                    dirs_dict = dirs_dict[2:]
                    result = self.__registerSubDirs(dirs_dict,dirs_meta)
                    if result:
                        create_round = 1
                else:
                    dir_exists = self.__dirExists(dir_data_mc,dir_bossVer)
                    if not dir_exists:
                        dirs_dict = dirs_dict[3:]
                        result = self.__registerSubDirs(dirs_dict,dirs_meta)
                        if result:
                            create_round = 1
                    else:
                        dir_exists = self.__dirExists(dir_eventType,dir_data_mc)
                        if not dir_exists:
                            dirs_dict = dirs_dict[4:]
                            result = self.__registerSubDirs(dirs_dict,dirs_meta)
                            if result:
                                create_round = 1
                        else:
                            dir_exists = self.__dirExists(dir_round,dir_eventType)
                            if not dir_exists:
                                dirs_dict = dirs_dict[5:]
                                result = self.__registerSubDirs(dirs_dict,dirs_meta)
                                if result:
                                    create_round = 1
                            else:
                                create_round = 1
        
        if create_round:
            if metaDict['streamId'] != "stream0":
                dir_exists = self.__dirExists(dir_streamId,dir_round)
                if not dir_exists:
                    if self.__registerDir(dir_streamId)['OK']:
                        result = self.__registerDirMetadata(dir_streamId,{'streamId':metaDict['streamId']})
                        if result['OK']:
                            result = self.__registerDirMetadata(dir_streamId,lastDirMetaDict)
                            if result['OK']:
                                creation_OK = 1
                else:
                    creation_OK = 2
            else:
                result = self.__registerDirMetadata(dir_round,lastDirMetaDict)
                if result['OK']:
                    creation_OK = 1
    
        if (creation_OK==1)|(creation_OK==2):
            if metaDict['streamId'] == "stream0":
                return dir_round
            else:   
                return dir_streamId
##########################################################################################
    #dir options
    def removeDir(self,dir):
        """remove the dir include files and subdirs
        """
        result = self.client.listDirectory(dir)
        if result['OK']:
            if not result['Value']['Successful'][dir]['Files'] and not result['Value']['Successful'][dir]['SubDirs']:
                #print 'no file and subDirs in this dir'
                self.client.removeDirectory(dir)
                return S_OK()
            else:
                if result['Value']['Successful'][dir]['Files']:
                    for file in result['Value']['Successful'][dir]['Files']:
                        self.client.removeFile(file)
                else:
                    for subdir in result['Value']['Successful'][dir]['SubDirs']:
                        self.removeDir(subdir)
                    self.removeDir(dir)

    def listDir(self,dir):
        """list the files under the given DFC dir"""
        fileList = []
        result = self.client.listDirectory(dir)
        if result['OK']:
          if result['Value']['Successful'][dir]['Files']:
            fileList = result['Value']['Successful'][dir]['Files'].keys()
            fileList.sort()
        else:
          print "no files under this dir"
        return fileList 

    def getDirMetaVal(self,dir):
      """list the registed metadata value of the given dir"""
      result = self.client.getDirectoryMetadata(dir)
      if result['OK']:
        return result['Value']
      else:
        print "Failed to get meta Value of the directory"
        return {}


    #################################################################################
    # meta fields operations
    #
    def addNewFields(self,fieldName,fieldType,metaType='-d'):
      """add new fields,if metaType is '-f',add file field,
        fileType is datatpye in MySQL notation
      """
      result = self.client.addMetadataField(fieldName,fieldType,metaType)
      if not result['OK']:
        return S_ERROR(result)
      else:
        return S_OK()

    def deleteMetaField(self,fieldName):
      """delete a exist metafield"""
      result = self.client.deleteMetadataField(fieldName)
      if not result['OK']:
        return S_ERROR(result)
      else:
        return S_OK()

    def getAllFields(self):
        """get all meta fields,include file metafield and dir metafield.
        """
        result = self.client.getMetadataFields()
        if not result['OK']:
          return S_ERROR(result['Message'])
        else:
          return result['Value']

    def registerFileMetadata(self,lfn,metaDict):

        """Add file level metadata to an entry
           True for success, False for failure
           (maybe used to registerNewMetadata
           Example:
           >>>lfn = '/bes/File/psipp/6.6.1/data/all/exp1/run_0011414_All_file001_SFO-1'
           >>>entryDict = {'runL':1000,'runH':898898}
           >>>badger.registerFileMetadata(lfn,entryDict)
           True
        """
        fc = self.client
        result = fc.setMetadata(lfn,metaDict)
        if result['OK']:
            return S_OK() 
        else:
            print 'Error:%s'%(result['Message'])
            return S_ERROR(result['Message'])
    #####################################################################
    # File Options
    def registerFile(self,lfn,dfcAttrDict):
        """Register a new file in the DFC.
        
        """
        #TODO:need more tests,if directory of file doesn't exist,
        #addFile will create it without setting any metadata(lin lei)
        #need to check whether directory of file exists in dfc?(lin lei) 
        #pass
        fc = self.client
        result = fc.addFile({lfn:dfcAttrDict})
        if result['OK']:
            if result['Value']['Successful']:
                if result['Value']['Successful'].has_key(lfn):
                    return S_OK()
            elif result['Value']['Failed']:
                if result['Value']['Failed'].has_key(lfn):
                    print 'Failed to add this file:',result['Value']['Failed'][lfn]
                    return S_ERROR()
        else:
            print 'Failed to add this file :',result['Message']
            return S_ERROR()
        # need to register file (inc. creating appropriate directory
        # if it doesn't already exist; and register metadata for that
        # file / directory
        # Q: how / where to pass the metadata?

    def getFileMetaVal(self,lfn):
      """get the File Meta Value of given file

      """
      result = self.client.getFileUserMetadata(lfn)
      if result['OK']:
        return result['Value']
      else:
        print "Failed to get meta Value of this file"
        return {}
    
    def reCalcCount(self,fileList,plus=True):
      """calculate the value of metadata 'count',when a file contain in a dataset
      count+1,when del a dataset,then all file in this dataset count -1
      default plus=True,means count+1,if count-1,set plus=False 
      return the value of count, count = -1 means error.
      NOTE:this function should only be called when create or delete a dataset.
      """
      countDict = {}
      if type(fileList)!=type([]):
        fileList = [fileList]
      for file in fileList:
        result =  self.getFileMetaVal(file)
        if len(result)!=0:
          count = result['count']
          if plus:
            count +=1
          else:
            if count>0:
              count -=1
          cDict = {'count':count}
          self.registerFileMetadata(file,cDict)
          countDict[file] = count
        else:
          print "Failed reCalculate value of count of file %s"%file
          countDict[file] = -1 

      return countDict

    def removeFile(self,lfn):
        """remove file on DFC
        """
        result = self.client.removeFile(lfn)
        if not result['OK']:
          return S_ERROR(result)
        else:
          return S_OK()

    def getPFN(self,lfn):
        """get replicas by lfn"""
        result = self.client.getReplicas(lfn)
        #print result
        if not result['OK']:
          return S_ERROR(result)
        else:
          return S_OK(result['Value']['Successful'][lfn]['IHEPD-USER'])

    def getSize(self,lfns):
        """get the size of the given lfn"""
        result = self.client.getFileSize(lfns)
        if result['OK']:
          if result['Value']['Successful']:
            retVal= result['Value']['Successful']
        else:
          retVal = {} 
        return retVal 

    def getFilesByMetadataQuery(self, query):
        """Return a list of LFNs satisfying given query conditions.

           Example usage:
           >>> brunH_GT_29756adger.getFilesByMetadataQuery('resonance=jpsi bossVer=6.5.5 round=exp1')
           ['/bes/File/jpsi/6.5.5/data/all/exp1/file1', .....]

        """
        #TODO: checking of output, error catching


        fc = self.client
        #TODO: calling the FileCatalog CLI object and its private method
        # is not a good way of doing this! but use it to allow construction of
        # the query meantime, until createQuery is made a public method
        cli = FileCatalogClientCLI(fc)
        metadataDict = cli._FileCatalogClientCLI__createQuery(query)
        result = fc.findFilesByMetadata(metadataDict,'/')
        if result['OK']:
            lfns = fc.findFilesByMetadata(metadataDict,'/')['Value']
            lfns.sort()
            return lfns
        else:
            print "ERROR: No files found which match query conditions."
            return None

    def uploadAndRegisterFiles(self,fileList,SE='IHEPD-USER',guid=None,ePoint=''):
        """upload a set of files to SE and register it in DFC.
        user input the directory of localfile.
        argument:
          ePoint is the energy point,for scan data
        we can treat localDir as a kind of datasetName.
        """          

        result_OK = 1
        errorList = []
        #fileList = self.getFilenamesByLocaldir(localDir)
        for fullpath in fileList:
          #get the attributes of the file
          fileAttr = self.__getFileAttributes(fullpath)
          if len(fileAttr) ==0:
            print "failed to get file %s attributes"%fullpath
            return S_ERROR("failed to get file attributes")
          #create dir and set dirMetadata to associated dir
          lastDir = self.registerHierarchicalDir(fileAttr,rootDir='/bes')
          dirMeta = self.getDirMetaVal(lastDir)
          if not (dirMeta.has_key("jobOptions") or dirMeta.has_key("description")):
            lastDirMetaDict = {}
            lastDirMetaDict['jobOptions'] = fileAttr['jobOptions']
            lastDirMetaDict['description'] = fileAttr['description']
            try:
              self.__registerDirMetadata(lastDir,lastDirMetaDict)
            except:
              pass
          if len(ePoint):
            lastDir = lastDir + os.sep + ePoint
          lfn = lastDir + os.sep + fileAttr['LFN']
          #upload and register file. 
          dirac = Dirac()
          result = dirac.addFile(lfn,fullpath,SE,guid,printOutput=True)
          #register file metadata
          if not result['OK']:
            print 'ERROR %s'%(result['Message'])
            #return S_ERROR(result['Message']) 
            errorList.append(fullpath)
            result_OK = 0
          else:
            result = self.__registerFileMetadata(lfn,fileAttr)
            if not result['OK']:
              result_OK = 0
              print "failed to register file metadata"
        if result_OK:
          return S_OK()
        else:
          return S_ERROR(errorList)

    def downloadFilesByFilelist(self,fileList,destDir=''):
        """downLoad a set of files form SE.
        use getFilesByFilelist() get a list of lfns and download these files.
        fileList get from function getFilesByDatesetName()

           Example usage:
           >>>badger.downloadFilesByFilelist(fileList)
        """
        errorDict = {}
        dirac = Dirac()
        #fileList = self.getFilesByDatasetName(dataset_name)
        for lfn in fileList:
          result = dirac.getFile(lfn,destDir,printOutput = False)
          if not result['OK']:
            errorDict[lfn] = result['Message']
        if errorDict:
          serr = S_ERROR()
          serr["errorDict"] = errorDict
          return serr
        else:
          return S_OK("File download successfully.") 

    ####################################################################
    # dataset functions
    #
    def registerDataset(self, datasetName,path,conditions):
        """Register a new dataset in DFC. Takes dataset name and string with
           conditions for new dataset as arguments.
           datasetname format:  
           resonance_BossVer_eventtype_round_runL_runH_stream0_datatype
           resonance_BossVer_eventtype_round_runL_runH_streamID_datatype
           type(conditions) is str,like "resonance=jpsi bossVer=655 round=round1"
        """
        fc = self.client
        cli = FileCatalogClientCLI(fc)
        metadataDict = cli._FileCatalogClientCLI__createQuery(conditions)
        metadataDict['Path'] = path 
        result = fc.addDataset(datasetName, metadataDict)
        if not result['OK']:
            print ("Error: %s" % result['Message'])
            return S_ERROR()
        else:
            print "Added dataset %s with conditions %s" % (datasetName, conditions)
            return S_OK()
        
    def getDatasetDescription(self, datasetName):
        """Return a string containing a description of metadata with which 
           the given dataset was defined.
           Example usage:
           >>> result = badger.getDatasetDescription('psipp_661_data_all_exp2')
        """
        result = self.client.getDatasetParameters(datasetName)
        if not result['OK']:
          print "ERROR: failed to get status of dataset:", result['Message']
        else:
          parDict = result['Value']
        for par,value in parDict.items():
          print par.rjust(20),':',value 

    def getDatasetMetadata(self, datasetName):
        """Return a dict containing a description of metadata with which 
           the given dataset was defined.
           Example usage:
           >>> result = badger.getDatasetMetadata('psipp_661_data_all_exp2')
        """
        result = self.client.getDatasetParameters(datasetName)
        if not result['OK']:
          print "ERROR: failed to get status of dataset:", result['Message']
          return S_ERROR(result['Message'])
        else:
          parDict = result['Value']
          return S_OK(parDict)

    def removeDataset(self,datasetName):
        """remove a dataset
        """
        result = self.client.removeDataset(datasetName)
        if not result['OK']:
          print "ERROR: failed to remove dataset:", result['Message']
        else:
          print "Successfully removed dataset", datasetName

    def checkDataset(self,datasetName):
        """ check if the dataset parameters are still valid
        """
        result = self.client.checkDataset( datasetName )
        if not result['OK']:
          print "ERROR: failed to check dataset:", result['Message']
        else:
          changeDict = result['Value']
          if not changeDict:
            print "Dataset is not changed"
          else:
            print "Dataset changed:"
            for par in changeDict:
              print "   ",par,': ',changeDict[par][0],'->',changeDict[par][1]
    def updateDataset(self,datasetName):
          """ Update the given dataset parameters
          """
          result = self.client.updateDataset( datasetName )
          if not result['OK']:
            print "ERROR: failed to update dataset:", result['Message']
          else:
            print "Successfully updated dataset", datasetName            

    def freezeDataset(self,datasetName):
        """ Freeze the given dataset
        """
        result = self.client.freezeDataset( datasetName )
        if not result['OK']:
          print "ERROR: failed to freeze dataset:", result['Message']
        else:
          print "Successfully frozen dataset", datasetName      

    def releaseDataset(self,datasetName):
        """ Release the given dataset
        """
        result = self.client.releaseDataset( datasetName )
        if not result['OK']:
          print "ERROR: failed to release dataset:", result['Message']
        else:
          print "Successfully released dataset", datasetName    

    def getFilesByDatasetName(self, datasetName):
        """Return a list of LFNs in the given dataset.
           
           Example usage:
           >>> badger.getFilesByDatasetName('psipp_661_data_all_exp2')
           ['/bes/File/psipp/6.6.1/data/all/exp2/file1', .....]
        """

        fc = self.client
        result = returnSingleResult(fc.getDatasetFiles(datasetName))
        if result['OK']:
          lfns = result['Value']
          lfns.sort()
          return S_OK(lfns)
        else:
          print "ERROR: Dataset", datasetName," not found"
          return S_ERROR(result)
            

    def listDatasets(self):
        """list the exist dataset"""
        datasetName = ''
        result = self.client.getDatasets( datasetName )
        if not result['OK']:
          print "ERROR:failed to get datasets"
          return
        
        datasetDict = result['Value']
        for dName in datasetDict.keys():
          print dName




    def checkDatasetIntegrity():
        pass





#!/usr/bin/env python
# -*- coding:utf-8 -*-
# author: lintao

import subprocess
import time
import select
import sys

import DIRAC
from DIRAC import gLogger

class ITransferWorker(object):
  """
  This is an Interface.
  """

  def __init__(self):
    self._proc = None
    self._buffer = ""

  @property
  def proc(self):
    return self._proc

  @property
  def info(self):
    return self._info

  def get_retcode(self):
    return self.proc.poll()

  def create_popen(self, info):
    cmd = self.build_cmd(info)
    gLogger.info("The info to create a command:")
    gLogger.info(info)
    gLogger.info("build command:")
    gLogger.info(cmd)

    self._info = info
    self._proc = subprocess.Popen(cmd, 
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE
        )
    # Make sure that the stdout is to PIPE.

  def build_cmd(self, info):
    raise NotImplementedError

  def handle_exit(self, returncode):
    raise NotImplementedError

  def handle_stream(self, stream):
    buffer_line = ""
    try:
      for line in stream:
        buffer_line += self.handle_line(line)
    except:
      pass
    finally:
      return buffer_line

  def handle_line(self, line):
    raise NotImplementedError

  def handle_waiting(self):
    if not self._proc:
      return

    r,w,x = select.select( [self._proc.stdout, self._proc.stderr], 
                           [], 
                           [self._proc.stdout, self._proc.stderr]
                           , 1 )
    for out in r:
      # ready for reading
      self._buffer += self.handle_stream( out )
    for out in x:
      # exceptional condition
      self._buffer += self.handle_stream( out )

class DemoTransferWorker(ITransferWorker):

  # Interface

  def build_cmd(self, info):
    return info

  def handle_exit(self, returncode):
    if returncode is None:
      return
    if returncode != 0:
      print "some error happens."
      print "work done"

  def handle_line(self, line):
    return line


if __name__ == "__main__":

  dtw = DemoTransferWorker()

  dtw.create_popen(["sleep", "5"])

  returncode = dtw.get_retcode()
  while returncode is None:
    time.sleep(0.2)
    dtw.handle_waiting()
    returncode = dtw.get_retcode()
  else:
    dtw.handle_exit(returncode)

  print "Work Done."





''' CacheFeederAgent

  This agent feeds the Cache tables with the outputs of the cache commands.

'''

from DIRAC                                                      import S_OK
from DIRAC.AccountingSystem.Client.ReportsClient                import ReportsClient
from DIRAC.Core.Base.AgentModule                                import AgentModule
from DIRAC.Core.DISET.RPCClient                                 import RPCClient
from DIRAC.Core.LCG.GOCDBClient                                 import GOCDBClient
from DIRAC.ResourceStatusSystem.Client.ResourceStatusClient     import ResourceStatusClient
from DIRAC.ResourceStatusSystem.Command                         import CommandCaller
from DIRAC.ResourceStatusSystem.Utilities                       import Utils
ResourceManagementIHEPClient = getattr( Utils.voimport( 'DIRAC.ResourceStatusSystem.Client.ResourceManagementIHEPClient' ), 'ResourceManagementIHEPClient' )

__RCSID__ = '$Id:  $'
AGENT_NAME = 'ResourceStatus/CacheFeederIHEPAgent'

class CacheFeederIHEPAgent( AgentModule ):
  '''
  The CacheFeederAgent feeds the cache tables for the client and the accounting.
  It runs periodically a set of commands, and stores it's results on the
  tables.
  '''

  # Too many public methods
  # pylint: disable=R0904

  def __init__( self, *args, **kwargs ):

    AgentModule.__init__( self, *args, **kwargs )

    self.commands = {}
    self.clients = {}

    self.cCaller = None
    self.rmClient = None

  def initialize( self ):

    self.am_setOption( 'shifterProxy', 'DataManager' )

    self.rmClient = ResourceManagementIHEPClient()

    self.commands[ 'JobIHEP' ] = [ { 'JobIHEP' : {} } ]
    self.commands[ 'StorageIHEP' ] = [ { 'StorageIHEP' : {} } ]
    self.commands[ 'WorkNodeIHEP' ] = [ { 'WorkNodeIHEP' : {} } ]

    # Reuse clients for the commands
    self.clients[ 'GOCDBClient' ] = GOCDBClient()
    self.clients[ 'ReportGenerator' ] = RPCClient( 'Accounting/ReportGenerator' )
    self.clients[ 'ReportsClient' ] = ReportsClient()
    self.clients[ 'ResourceStatusClient' ] = ResourceStatusClient()
    self.clients[ 'ResourceManagementIHEPClient' ] = ResourceManagementIHEPClient()
    self.clients[ 'WMSAdministrator' ] = RPCClient( 'WorkloadManagement/WMSAdministrator' )

    self.cCaller = CommandCaller

    return S_OK()

  def loadCommand( self, commandModule, commandDict ):

    commandName = commandDict.keys()[ 0 ]
    commandArgs = commandDict[ commandName ]

    commandTuple = ( '%sCommand' % commandModule, '%sCommand' % commandName )
    commandObject = self.cCaller.commandInvocation( commandTuple, pArgs = commandArgs,
                                                    clients = self.clients )

    if not commandObject[ 'OK' ]:
      self.log.error( 'Error initializing %s' % commandName )
      return commandObject
    commandObject = commandObject[ 'Value' ]

    # Set master mode
    commandObject.masterMode = True

    self.log.info( '%s/%s' % ( commandModule, commandName ) )

    return S_OK( commandObject )


  def execute( self ):

    for commandModule, commandList in self.commands.items():

      self.log.info( '%s module initialization' % commandModule )

      for commandDict in commandList:

        commandObject = self.loadCommand( commandModule, commandDict )
        if not commandObject[ 'OK' ]:
          self.log.error( commandObject[ 'Message' ] )
          continue
        commandObject = commandObject[ 'Value' ]

        results = commandObject.doCommand()

        if not results[ 'OK' ]:
          self.log.error( 'Failed to execute command', '%s: %s' % ( commandModule, results[ 'Message' ] ) )
          continue
        results = results[ 'Value' ]

        if not results:
          self.log.info( 'Empty results' )
          continue

        self.log.verbose( 'Command OK Results' )
        self.log.verbose( results )

    return S_OK()

################################################################################
# EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF



""" StatusEvaluator

  StatusEvaluator is used to evaluate elements' SAM status depending on the
  SAM tests results. The SAM status includes resource SAM status and site SAM
  status. Firstly, integrate tests results and get resource SAM status. Then judge
  site SAM status depending on the status of resources which belong to the site.
"""

from datetime import datetime
from DIRAC                                                         import S_OK, S_ERROR
from IHEPDIRAC.ResourceStatusSystem.Client.ResourceManagementIHEPClient import ResourceManagementIHEPClient


__RCSID__ = '$Id:  $'


class StatusEvaluator(object):
  """ StatusEvaluator
  """

  def __init__(self, apis):
    """ Constructor

    examples:
      >>> sites = { 'CLUSTER' : [ 'CLUSTER.USTC.cn' ],
                              'GRID' : [ 'GRID.JINR.ru' ],
                              'CLOUD' : [ ''CLOUD.IHEP-OPENSTACK.cn' ] }
      >>> evaluator = StatusEvaluator( sites )

    :Parameters:
       **sites** - `dict`
         the sites to evaluate SAM status. The sites is grouped by domain.
    """

    if "ResourceManagementIHEPClient" in apis:
      self.rmClient = apis[ "ResourceManagementIHEPClient" ]
    else:
      self.rmClient = ResourceManagementIHEPClient()


  def __storeResourceStatus( self, resDict ):
    storeRes = self.rmClient.addOrModifyResourceSAMStatus(
                                                          resDict[ 'VO' ],
                                                          resDict[ 'ElementName' ],
                                                          resDict[ 'ElementType' ],
                                                          resDict[ 'Tests' ],
                                                          resDict[ 'Status' ],
                                                          resDict[ 'LastCheckTime' ]
                                                          )
    if not storeRes[ 'OK' ]:
      return storeRes
    return S_OK()


  def __storeSiteStatus( self, resDict ):
    storeRes = self.rmClient.addOrModifySiteSAMStatus(
                                                      resDict[ 'VO' ],
                                                      resDict[ 'Site' ],
                                                      resDict[ 'SiteType' ],
                                                      resDict[ 'Status' ],
                                                      resDict[ 'CEStatus' ],
                                                      resDict[ 'SEStatus' ],
                                                      resDict[ 'LastCheckTime' ]
                                                      )
    if not storeRes[ 'OK' ]:
      return storeRes
    return S_OK()


  def __resourceStatusRule( self, statusList ):
    if 'Bad' in statusList:
      return 'Bad'
    if 'Unknown' in statusList:
      return 'Unknown'
    if 'Busy' in statusList:
      return 'Busy'
    if 'OK' in statusList:
      return 'OK'
    return ''


  def __siteStatusRule( self, ceStatusList, seStatus ):
    if 'OK' in ceStatusList:
      ceStatus = 'OK'
    elif 'Busy' in ceStatusList:
      ceStatus = 'Busy'
    elif 'Bad' in ceStatusList:
      ceStatus = 'Bad'
    else:
      ceStatus = 'Unknown'

    if not seStatus:
      status = ceStatus
    else:
      if 'Bad' == seStatus:
        status = 'Bad'
      else:
        status = ceStatus

    return ( status, ceStatus, seStatus )


  def evaluateResourceStatus( self, elementDict, testResults, vo = None, lastCheckTime = None ):
    vo = vo or 'all'
    lastCheckTime = lastCheckTime or datetime.utcnow().replace( microsecond = 0 )
    elementName = elementDict[ 'ElementName' ]
    elementType = elementDict[ 'ElementType' ]
    tests = ','.join( testResults.keys() )
    status = self.__resourceStatusRule( testResults.values() )

    resDict = { 'ElementName' : elementName,
                        'VO' : vo,
                        'ElementType' : elementType,
                        'Tests' : tests,
                        'Status' : status,
                        'LastCheckTime' : lastCheckTime }

    storeRes = self.__storeResourceStatus( resDict )
    if not storeRes[ 'OK' ]:
      return S_ERROR( 'Failed to store resource SAM status.' )
    return S_OK( status )


  def evaluateSiteStatus( self, site, ceStatusList, seStatus = None, vo = None, lastCheckTime = None ):
    vo = vo or 'all'
    lastCheckTime = lastCheckTime or datetime.utcnow().replace( microsecond = 0 )
    siteType = site.split( '.' )[ 0 ]
    status, ceStatus, seStatus = self.__siteStatusRule( ceStatusList, seStatus )

    resDict = { 'Site' : site,
                         'VO' : vo,
                         'SiteType' : siteType,
                         'Status' : status,
                         'CEStatus' : ceStatus,
                         'SEStatus' : seStatus,
                         'LastCheckTime' : lastCheckTime }

    storeRes = self.__storeSiteStatus( resDict )
    if not storeRes[ 'OK' ]:
      S_ERROR( 'Failed to store site SAM status.' )
    return S_OK( status )



import subprocess

from DIRAC import gLogger

from IHEPDIRAC.Badger.private.output.getfile.GetFile import GetFile
from IHEPDIRAC.Badger.private.output.getfile.LocalMount import LocalMount

class CpGetFile(LocalMount, GetFile):
  def __init__(self):
    super(CpGetFile, self).__init__()

  def _downloadSingleFile(self, remotePath, localPath):
    gLogger.debug('cp from %s to %s' % (remotePath, localPath))

    args = ['cp', remotePath, localPath]
    p = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out, err = p.communicate()
    ret = p.returncode

    return ret == 0



""" BOSSTest

  A test class to test BOSS is ok or not.

"""

from IHEPDIRAC.ResourceStatusSystem.SAM.SAMTest.CEBaseTest import CEBaseTest


__RCSID__ = '$Id: $'


class BOSSTest( CEBaseTest ):
  """
    BOSSTest is used to test whether BOSS is fine to run jobs.
  """

  def __init__( self, args = None, apis = None ):
    super( BOSSTest, self ).__init__( args, apis )


  @staticmethod
  def _judge( log ):
    """
      judge the BOSS test status.
    """

    dry = []
    lines = log.split( '\n' )
    for line in lines:
      line = line.split( ' ', 1 )
      if line[ 0 ] in [ 'DstHltMaker', 'ApplicationMgr' ]:
        line[ 1 ] = line[ 1 ].strip()
        dry.append( line )

    applicationMgrSuccess = 0
    dstHltMakerSuccess = 0
    expectedApplicationMng = 'INFO Application Manager Terminated successfully'
    expectedDstHltMaker = 'SUCCESS 50 events are converted.'
    for i in dry:
      if i[ 0 ] == 'ApplicationMgr' and i[ 1 ] == expectedApplicationMng:
        applicationMgrSuccess += 1
      if i[ 0 ] == 'DstHltMaker' and i[ 1 ] == expectedDstHltMaker:
        dstHltMakerSuccess += 1
    if applicationMgrSuccess == 2 and dstHltMakerSuccess == 1:
      return 'OK'
    return 'Bad'



import os
from DIRAC import S_OK, S_ERROR, gConfig
from DIRAC.Interfaces.API.Dirac import Dirac
from DIRAC.FrameworkSystem.Client.ProxyManagerClient import gProxyManager
from DIRAC.ConfigurationSystem.Client.Helpers import Registry


def getProxyByVO( user, vo ):
  group = vo.lower() + '_user'

  userDN = Registry.getDNForUsername( user )
  if not userDN[ 'OK' ]:
    return S_ERROR( 'Cannot discover DN for user %s: %s' % ( user, userDN[ 'Message' ] ) )
  userDN = userDN[ 'Value' ][ 0 ]

  chain = gProxyManager.downloadVOMSProxy( userDN, group )
  if not chain[ 'OK' ]:
    return S_ERROR( 'Proxy file cannot be retrieved: %s' % chain[ 'Message' ] )
  chain = chain[ 'Value' ]

  proxyPath = "%s/proxy.%s.%s" % ( os.getcwd(), user, group )
  result = chain.dumpAllToFile( proxyPath )
  if not result[ 'OK' ]:
    return S_ERROR( 'Proxy file cannot be written to %s: %s' % ( proxyPath, result[ 'Message' ] ) )

  return S_OK( proxyPath )


def getSiteForCE( ce ):
  _basePath = 'Resources/Sites'

  domains = gConfig.getSections( _basePath )[ 'Value' ]
  for domain in domains:
    sites = gConfig.getSections( '%s/%s' % ( _basePath, domain ) )[ 'Value' ]
    for site in sites:
      ces = gConfig.getValue( '%s/%s/%s/CE' % ( _basePath, domain, site ) , '' ).split( ',' )
      ces = map(lambda str : str.strip(), ces);
      if ce in ces:
        return site


def getSitesForSE( se ):
  _basePath = 'Resources/Sites'

  seSites = []
  domains = gConfig.getSections( _basePath )[ 'Value' ]
  for domain in domains:
    sites = gConfig.getSections( '%s/%s' % ( _basePath, domain ) )[ 'Value' ]
    for site in sites:
      ses = gConfig.getValue( '%s/%s/%s/SE' % ( _basePath, domain, site ), '' ).split( ',' )
      ses = map(lambda str : str.strip(), ses);
      if se in ses:
        seSites.append(site)

  return seSites


def getSiteVO( siteName ):
  _basePath = 'Resources/Sites'

  res = Registry.getVOs()
  if res[ 'OK' ]:
    allVOs = res[ 'Value' ]
  else:
    allVOs = [ 'bes', 'cepc', 'juno' ]

  domain = siteName.split( '.' )[ 0 ]
  if domain == 'CLOUD':
    vos = []
    clouds = gConfig.getSections( '%s/CLOUD/%s/Cloud/' % ( _basePath, siteName ) )
    if clouds[ 'OK' ]:
      for cloud in clouds[ 'Value' ]:
        images = gConfig.getSections( '%s/CLOUD/%s/Cloud/%s/Images' % ( _basePath, siteName, cloud ) )
        if images[ 'OK' ]:
          for image in images[ 'Value' ]:
            vo = gConfig.getValue( '%s/CLOUD/%s/Cloud/%s/Images/%s/VO' % ( _basePath, siteName, cloud, image ) )
            if vo:
              vos.append( vo )
    if vos:
      return list( set( vos ) )

  else:
    vos = gConfig.getValue( '%s/%s/%s/VO' % ( _basePath, domain, siteName ) )
    if vos is None:
      ces = gConfig.getSections( '%s/%s/%s/CEs' % ( _basePath, domain, siteName ) )
      if ces[ 'OK' ]:
        ces = ces[ 'Value' ]
        vos = gConfig.getValue( '%s/%s/%s/CEs/%s/VO' % ( _basePath, domain, siteName, ces[ 0 ] ) )
        if vos is None:
          queues = gConfig.getSections( '%s/%s/%s/CEs/%s/Queues' % ( _basePath, domain, siteName, ces[ 0 ] ) )
          if queues[ 'OK' ]:
            queues = queues[ 'Value' ]
            vos = gConfig.getValue( '%s/%s/%s/CEs/%s/Queues/%s/VO' % ( _basePath, domain, siteName, ces[ 0 ], queues[ 0 ] ) )
    if vos:
      vos = [ vo.strip() for vo in vos.split( ',' ) ]
      return vos

  return allVOs



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# author: lintao

import subprocess

import DIRAC
from DIRAC import gLogger

class TransferFactory(object):
  PROTOCOL = ["DIRACFTS", "DIRACDMS"]

  def generate(self, protocol, info):
    gLogger.info("Load Module:")
    gLogger.info("IHEPDIRAC.TransferSystem.Agent.helper.TransferFactory.TransferBy%s"%(protocol))

    try:
      mod = __import__("IHEPDIRAC.TransferSystem.Agent.helper.TransferFactory.TransferBy%s"%(protocol),
          globals(),
          locals(),
          ["%sTransferWorker"%(protocol)]
          )
      TR = getattr(mod, "%sTransferWorker"%(protocol))
    except Exception as e:
      gLogger.error('Load transfer protocol "%s" error: %s' % (protocol, e))
      return None

    tr = TR()
    tr.create_popen(info)
    return tr


gTransferFactory = TransferFactory()




import json

from WebAppDIRAC.Lib.WebHandler import WebHandler, WErr, WOK, asyncGen
from DIRAC import gConfig, S_OK, S_ERROR, gLogger
from DIRAC.Core.DISET.RPCClient import RPCClient
from IHEPDIRAC.WorkloadManagementSystem.Client.TaskClient import TaskClient

STATUS_ICON_MAP = {
  'Init' : 'Checking',
  'Ready' : 'Waiting',
  'Processing' : 'Running',
  'Finished' : 'Done',
  'Expired' : 'Bad',
  'Removed' : 'Deleted'
}

class TaskManagerHandler(WebHandler):

    AUTH_PROPS = "authenticated"

    @asyncGen
    def web_getSelectionData(self):
      sData = self.getSessionData()

      taskClient = TaskClient()

      callback = {}

      user = sData["user"]["username"]
      group = sData["user"]["group"]

      if user == "Anonymous":
        callback["status"] = [["Insufficient rights"]]
      elif 'JobAdministrator' in sData['user']['properties']:
        callback["status"] = [['Init'],['Ready'],['Processing'],['Finished'],['Expired'],['Removed']]
      elif 'NormalUser' in sData['user']['properties']:
        callback["status"] = [['Init'],['Ready'],['Processing'],['Finished'],['Expired']]

      if user == "Anonymous":
        callback["owner"] = [["Insufficient rights"]]
      elif 'NormalUser' in sData['user']['properties']:
        owner = [[user]]
      else:
        result = yield self.threadTask( taskClient.getTaskOwners )
        if result["OK"]:
          owner = []
          if len( result["Value"] ) > 0:
            for i in result["Value"]:
              owner.append( [str( i )] )
          else:
            owner = [["Nothing to display"]]
        else:
            gLogger.error( "getTaskOwners() return error: %s" % result["Message"] )
            owner = [["Error happened on service side"]]
      callback["owner"] = owner

      if user == "Anonymous":
        callback["ownerGroup"] = [["Insufficient rights"]]
      elif 'NormalUser' in sData['user']['properties']:
        ownerGroup = [[group]]
      else:
        result = yield self.threadTask( taskClient.getTaskOwnerGroups )
        if result["OK"]:
          ownerGroup = []
          if len( result["Value"] ) > 0:
            for i in result["Value"]:
              ownerGroup.append( [str( i )] )
          else:
            ownerGroup = [["Nothing to display"]]
        else:
            gLogger.error( "getTaskOwnerGroups() return error: %s" % result["Message"] )
            ownerGroup = [["Error happened on service side"]]
      callback["ownerGroup"] = ownerGroup

      self.finish( callback )

    @asyncGen
    def web_getTaskData( self ):
      taskClient = TaskClient()
      req = self.__request()

      condDict = req
      limit = self.numberOfJobs
      offset = self.pageNumber
      orderAttribute = self.globalSort[0][0] + ':' + self.globalSort[0][1]

      result = yield self.threadTask( taskClient.getTaskCount, condDict )
      if not result["OK"]:
        self.finish( {"success":"false", "result":[], "total":0, "error":result["Message"]} )
        return
      total = result['Value']

      result = yield self.threadTask( taskClient.listTask, condDict, limit, offset, orderAttribute )
      if not result["OK"]:
        self.finish( {"success":"false", "result":[], "total":0, "error":result["Message"]} )
        return

      fields = ['TaskID', 'TaskName', 'Status', 'Owner', 'OwnerGroup', 'OwnerDN', 'Site', 'JobGroup']
      values = []
      for data in result['Value']:
        value = {}
        for d in data:
          if d in fields:
            value[d] = data[d]
        value['IconStatus'] = STATUS_ICON_MAP[data['Status']]
        value['CreationTime'] = str( data['CreationTime'] )
        value['UpdateTime'] = str( data['UpdateTime'] )
        value['Progress'] = data['Progress']

        totalJobs = data['Progress'].get('Total', 0)
        deletedJobs = data['Progress'].get('Deleted', 0)
        value['Total'] = '%s/%s' % (totalJobs-deletedJobs, totalJobs)

        values.append( value )

      callback = {"success":"true", "result":values, "total":total, "date":None}
      self.finish( callback )

    @asyncGen
    def web_getTaskInfo( self ):
      taskClient = TaskClient()

      taskID = int( self.request.arguments["TaskID"][0] )

      result = yield self.threadTask( taskClient.getTaskInfo, taskID )
      if not result["OK"]:
        self.finish( {"success":"false", "result":'', "error":result["Message"]} )
        return

      allInfo = []
      for name, value in sorted(result['Value'].iteritems(), key=lambda d:d[0]):
        if type( value ) == type([]):
          value = ', '.join( value )
        allInfo.append( [name, str(value)] )

      callback = {"success":"true", "result":allInfo}
      self.finish( callback )

    @asyncGen
    def web_getTaskHistory( self ):
      taskClient = TaskClient()

      taskID = int( self.request.arguments["TaskID"][0] )

      result = yield self.threadTask( taskClient.getTaskHistories, taskID )
      if not result["OK"]:
        self.finish( {"success":"false", "result":'', "error":result["Message"]} )
        return

      histories = []
      for status, statusTime, description in result['Value']:
        histories.append( [status, str(statusTime), description] )

      callback = {"success":"true", "result":histories}
      self.finish( callback )

    @asyncGen
    def web_renameTask( self ):
      taskClient = TaskClient()

      taskID = int( self.request.arguments["TaskID"][0] )
      newName = self.request.arguments["NewName"][0]

      result = yield self.threadTask( taskClient.renameTask, taskID, newName )
      if not result["OK"]:
        self.finish( {"success":"false", "result":'', "error":result["Message"]} )
        return

      callback = {"success":"true", "result":result['Value']}
      self.finish( callback )

    @asyncGen
    def web_getTaskProgress( self ):
      taskClient = TaskClient()

      taskID = int( self.request.arguments["TaskID"][0] )

      result = yield self.threadTask( taskClient.getTaskProgress, taskID )
      if not result["OK"]:
        self.finish( {"success":"false", "result":'', "error":result["Message"]} )
        return

      statuses = ['Total', 'Done', 'Failed', 'Running', 'Waiting', 'Deleted']

      allProgress = []
      for status in statuses:
        allProgress.append( [status, result['Value'].get( status, 0 )] )

      callback = {"success":"true", "result":allProgress}
      self.finish( callback )

    @asyncGen
    def web_getTaskJobs( self ):
      taskClient = TaskClient()

      taskID = int( self.request.arguments["TaskID"][0] )

      result = yield self.threadTask( taskClient.getTaskJobs, taskID )
      if not result["OK"]:
        self.finish( {"success":"false", "result":'', "error":result["Message"]} )
        return

      callback = {"success":"true", "result":result['Value']}
      self.finish( callback )

    @asyncGen
    def web_getTaskJobList( self ):
      taskClient = TaskClient()

      taskID = int( self.request.arguments["TaskID"][0] )

      result = yield self.threadTask( taskClient.getTaskJobs, taskID )
      if not result["OK"]:
        self.finish( {"success":"false", "result":'', "error":result["Message"]} )
        return

      jobIDs = result['Value']

      RPC = RPCClient( "WorkloadManagement/JobMonitoring" )
      result = RPC.getJobsStatus( result['Value'] )
      if not result["OK"]:
        self.finish( {"success":"false", "result":'', "error":result["Message"]} )
        return

      jobStatuses = result['Value']
      jobList = []
      for jobID in jobIDs:
        if jobID in jobStatuses:
          jobList.append( [jobID, jobStatuses[jobID]['Status']] )
        else:
          jobList.append( [jobID, 'Deleted'] )

      callback = {"success":"true", "result":jobList}
      self.finish( callback )

    @asyncGen
    def web_getTaskJobsStatistics( self ):
      taskClient = TaskClient()

      taskID = int( self.request.arguments["TaskID"][0] )

      statistics = []

      statistics.append( ['==== Status ====', ''] )
      result = yield self.threadTask( taskClient.getTaskJobsStatistics, taskID, 'Status' )
      if result["OK"]:
        for s in result['Value']:
          statistics.append( [s, result['Value'].get( s, 0 )] )

      statistics.append( ['', ''] )

      statistics.append( ['==== Minor Status ====', ''] )
      result = yield self.threadTask( taskClient.getTaskJobsStatistics, taskID, 'MinorStatus' )
      if result["OK"]:
        for s in result['Value']:
          statistics.append( [s, result['Value'].get( s, 0 )] )

      statistics.append( ['', ''] )

      statistics.append( ['==== Application Status ====', ''] )
      result = yield self.threadTask( taskClient.getTaskJobsStatistics, taskID, 'ApplicationStatus' )
      if result["OK"]:
        for s in result['Value']:
          statistics.append( [s, result['Value'].get( s, 0 )] )

      statistics.append( ['', ''] )

      statistics.append( ['==== Site ====', ''] )
      result = yield self.threadTask( taskClient.getTaskJobsStatistics, taskID, 'Site' )
      if result["OK"]:
        for s in result['Value']:
          statistics.append( [s, result['Value'].get( s, 0 )] )

      statistics.append( ['', ''] )

      statistics.append( ['==== Job Group ====', ''] )
      result = yield self.threadTask( taskClient.getTaskJobsStatistics, taskID, 'JobGroup' )
      if result["OK"]:
        for s in result['Value']:
          statistics.append( [s, result['Value'].get( s, 0 )] )

      callback = {"success":"true", "result":statistics}
      self.finish( callback )

    @asyncGen
    def web_getTaskJobInfo( self ):
      taskClient = TaskClient()

      jobID = int( self.request.arguments["JobID"][0] )

      outFields = ['Info']
      result = yield self.threadTask( taskClient.getJobs, [jobID], outFields )
      if not ( result["OK"] and result['Value'] ):
        self.finish( {"success":"false", "result":'', "error":result["Message"]} )
        return

      info = json.loads( result['Value'][0][0] )

      allInfo = []
      for name, value in sorted(info.iteritems(), key=lambda d:d[0]):
        if type( value ) == type([]):
          value = ', '.join( value )
        allInfo.append( [name, str(value)] )

      callback = {"success":"true", "result":allInfo}
      self.finish( callback )

    @asyncGen
    def web_rescheduleTask( self ):
      taskClient = TaskClient()

      taskIDs = self.request.arguments["TaskID"][0].split( ',' )
      jobStatus = self.request.arguments["JobStatus"][0].split( ',' )
      if '' in taskIDs:
        taskIDs.remove( '' )
      if '' in jobStatus:
        jobStatus.remove( '' )
      taskIDs = [ int(taskID) for taskID in taskIDs ]

      for taskID in taskIDs:
        result = yield self.threadTask( taskClient.rescheduleTask, taskID, jobStatus )
        if not result["OK"]:
          self.finish( {"success":"false", "result":'', "error":result["Message"]} )
          return

      callback = {"success":"true", "result":taskIDs}
      self.finish( callback )

    @asyncGen
    def web_deleteTask( self ):
      taskClient = TaskClient()

      taskIDs = self.request.arguments["TaskID"][0].split( ',' )
      if '' in taskIDs:
        taskIDs.remove( '' )
      taskIDs = [ int(taskID) for taskID in taskIDs ]

      for taskID in taskIDs:
        result = yield self.threadTask( taskClient.deleteTask, taskID )
        if not result["OK"]:
          self.finish( {"success":"false", "result":'', "error":result["Message"]} )
          return

      callback = {"success":"true", "result":taskIDs}
      self.finish( callback )


    def __request( self ):
      self.pageNumber = 0
      self.numberOfJobs = 25
      self.globalSort = [["TaskID", "DESC"]]

      req = {}
      
      if self.request.arguments.has_key( "limit" ) and len( self.request.arguments["limit"][0] ) > 0:
        self.numberOfJobs = int( self.request.arguments["limit"][0] )
        if self.request.arguments.has_key( "start" ) and len( self.request.arguments["start"][0] ) > 0:
          self.pageNumber = int( self.request.arguments["start"][0] )
        else:
          self.pageNumber = 0

      if "TaskID" in self.request.arguments:
        taskids = list( json.loads( self.request.arguments[ 'TaskID' ][-1] ) )
        if len( taskids ) > 0:
          req['TaskID'] = taskids


      if "status" in self.request.arguments:
        status = list( json.loads( self.request.arguments[ 'status' ][-1] ) )
        if len( status ) > 0:
          req["Status"] = status

      if "owner" in self.request.arguments:
        owner = list( json.loads( self.request.arguments[ 'owner' ][-1] ) )
        if len( owner ) > 0:
          req["Owner"] = owner
      
      if "ownerGroup" in self.request.arguments:
        ownerGroup = list( json.loads( self.request.arguments[ 'ownerGroup' ][-1] ) )
        if len( ownerGroup ) > 0:
          req["OwnerGroup"] = ownerGroup
          
      if 'startDate' in self.request.arguments and len( self.request.arguments["startDate"][0] ) > 0:
        if 'startTime' in self.request.arguments and len( self.request.arguments["startTime"][0] ) > 0:
          req["FromDate"] = str( self.request.arguments["startDate"][0] + " " + self.request.arguments["startTime"][0] )
        else:
          req["FromDate"] = str( self.request.arguments["startDate"][0] )

      if 'endDate' in self.request.arguments and len( self.request.arguments["endDate"][0] ) > 0:
        if 'endTime' in self.request.arguments and len( self.request.arguments["endTime"][0] ) > 0:
          req["ToDate"] = str( self.request.arguments["endDate"][0] + " " + self.request.arguments["endTime"][0] )
        else:
          req["ToDate"] = str( self.request.arguments["endDate"][0] )

      if 'date' in self.request.arguments and len( self.request.arguments["date"][0] ) > 0:
        req["LastUpdate"] = str( self.request.arguments["date"][0] )

      if 'sort' in self.request.arguments:
        sort = json.loads( self.request.arguments['sort'][-1] )
        if len( sort ) > 0:
          self.globalSort = []
          for i in sort :
            self.globalSort += [[str( i['property'] ), str( i['direction'] )]]
      else:
        self.globalSort = [["TaskID", "DESC"]]

      gLogger.debug( "Request", str( req ) )
      return req



import os
import time
import imp

from DIRAC import gLogger

from IHEPDIRAC.Badger.private.output.MergeFile import MergeFile

class GetOutputHandler(object):
  def __init__(self, lfnList, method, localValidation, useChecksum):
    self.__lfnList = lfnList

    if type(method) is list:
      self.__method = self.__decideAvailableMethod(method)
      gLogger.debug('Decide available method:', (method, self.__method))
    else:
      self.__method = method

    try:
      self.__createGetFile()
    except Exception, e:
      gLogger.error('CreateGetFile error:', self.__method)
      raise Exception('Method could not be imported: %s' % self.__method)

    self.__getFile.setUseChecksum(useChecksum)
    self.__getFile.setLocalValidation(localValidation)

    self.__mergeFile = MergeFile()

  def getMethod(self):
    return self.__method

  def getAvailNumber(self):
    return len(self.__lfnAvailList)

  def checkRemote(self):
    allRemoteAttributes = self.__getFile.getAllRemoteAttributes(self.__lfnList)
    self.__lfnAvailList = [lfn for lfn in self.__lfnList if lfn in allRemoteAttributes and allRemoteAttributes[lfn]]

  def download(self, downloadDir, downloadCallback=None):
    count = {}

    for lfn in self.__lfnAvailList:
      result = self.__getFile.getFile(lfn, downloadDir)
      if result['status'] in count:
        count[result['status']] += 1
      else:
        count[result['status']] = 1

      if downloadCallback is not None:
        downloadCallback(lfn, result)

    return count

  def downloadAndMerge(self, downloadDir, mergeDir, mergeName, mergeExt, mergeMaxSize, removeDownload,
                       downloadCallback=None, mergeCallback=None, removeCallback=None):
    if self.__getFile.directlyRead() and removeDownload:
      ret = self.__mergeFromRemote(mergeDir, mergeName, mergeExt, mergeMaxSize, mergeCallback)
      if removeDownload and ret:
        self.__removeLocalDownloaded(downloadDir, removeCallback)
    else:
      count = self.download(downloadDir, downloadCallback)
      if mergeMaxSize > 0:
        if 'error' not in count or count['error'] == 0:
          ret = self.__mergeFromLocal(downloadDir, mergeDir, mergeName, mergeExt, mergeMaxSize, mergeCallback)
          if removeDownload and ret:
            self.__removeLocalDownloaded(downloadDir, removeCallback)


  def __createGetFile(self):
    getFileClassName = ''.join(w.capitalize() for w in self.__method.split('_')) + 'GetFile'
    getFileModuleName = 'IHEPDIRAC.Badger.private.output.getfile.%s' % getFileClassName

    try:
      self.__getFile = self.__loadClass(getFileModuleName, getFileClassName)
    except ImportError, e:
      raise Exception('Could not find method %s' % self.__method)

  def __decideAvailableMethod(self, methodList):
    for method in methodList:
      getFileClassName = ''.join(w.capitalize() for w in method.split('_')) + 'GetFile'
      getFileModuleName = 'IHEPDIRAC.Badger.private.output.getfile.%s' % getFileClassName

      try:
        getFile = self.__loadClass(getFileModuleName, getFileClassName)
      except ImportError, e:
        gLogger.debug('Could not find method:', self.__method)
        continue

      if getFile.available():
        return method

    return 'dfc'


  def __mergeFromLocal(self, downloadDir, mergeDir, mergeName, mergeExt, mergeMaxSize, mergeCallback):
    mergeList = self.__lfnAvailList[:]
    localMergeList = [self.__getFile.lfnToLocal(downloadDir, lfn) for lfn in mergeList]
    localMergeList.sort()
    return self.__mergeFile.merge(localMergeList, mergeDir, mergeName, mergeExt, mergeMaxSize, mergeCallback)

  def __mergeFromRemote(self, mergeDir, mergeName, mergeExt, mergeMaxSize, mergeCallback):
    mergeList = self.__lfnAvailList[:]
    remoteMergeList = [self.__getFile.lfnToRemote(lfn) for lfn in mergeList]
    remoteMergeList.sort()
    return self.__mergeFile.merge(remoteMergeList, mergeDir, mergeName, mergeExt, mergeMaxSize, mergeCallback)

  def __removeLocalDownloaded(self, downloadDir, removeCallback):
    for lfn in self.__lfnAvailList:
      localPath = self.__getFile.lfnToLocal(downloadDir, lfn)
      if os.path.isfile(localPath):
        os.remove(localPath)
        if removeCallback is not None:
          removeCallback(localPath)

  def __loadClass(self, moduleName, className):
    try:
      m = __import__(moduleName, globals(), locals(), [className])
    except ImportError, e:
      raise Exception('Could not from %s import %s' % (moduleName, className))
    return getattr(m, className)()




# -*- coding:utf-8 -*-
#data 13/07/26
#for data/all  name of file like run_0023454_All_file014_SFO-2.dst
#for data/skim & mc, we use new file naming rule,
#file name like resonance_eventType_streamId_runL_runH_*.dst

import os
import os.path
import string
import re

def get_module_dir():
  return os.path.dirname( os.path.abspath(__file__) )

from DIRAC.Core.Base import Script
from DIRAC import S_OK,S_ERROR
Script.parseCommandLine( ignoreErrors = True )
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
    
#judge format of file
class JudgeFormat(Exception):
    def __init__(self, format):
        self.format = format
    def __str__(self):
        return repr("the File's format is not ",self.format)

#type of srcformat is list,it includes many formats
def checkFormat(srcformat,file):
    flag = 0
    for format in srcformat:
        #if format of file is in srcformat
        if  file.endswith(format):
            flag = 1
    return flag
                
        
#dstfile like /bes3fs/offline/data/655-1/4040/dst/110504/run_0023474_All_file007_SFO-2.dst
def getLFN(dstfile,format=[".dst",".tag"]):
    flag = checkFormat(format,dstfile)

    if flag==0:
        raise JudgeFormat(format)
        return
    #split dstfile by "/",then get "lfn.dst"    
    items=dstfile.split("/")
    length=len(items)

    filename=items[length-1]
    
    #split "*.dst" by "."
    #get lfn
    #lfn = filename.split('.')[0]
    lfn = filename

    return lfn

#get size of dst file
def getFileSize(dstfile,format = [".dst",".tag"]):
    flag = checkFormat(format,dstfile)
    
    if flag==0:
        raise JudgeFormat(format)
        return
    
    if os.path.exists(dstfile):
        #get file's size
        return os.path.getsize(dstfile)


#lfn like resonance_eventType_streamId_runL_runH_*,get attributes:resonance,eventType,streamId,runL,runH 
#lfn like run_0009947_All_file001_SFO-1,get attribute runId
def splitLFN(lfn,type):
    result = {}
        
    items = lfn.split("_")

    if type == "all":
        if items[2] == "All":
            runId = string.atoi(items[1])
            return runId
    
    else:        
        result["resonance"] = filter(str.isalpha,items[0])
        result["streamId"] = items[1]
        result["runL"] = int(filter(str.isdigit,items[2]))
        result["runH"] = int(filter(str.isdigit,items[2]))
   
        return result
    


#get runIdList from JobOptions
def getRunIdList(jobOptions):
    result = {}    
    runIdList = []
    str1=jobOptions[0]
    pat = re.compile(r'RunIdList= {-\d+(,-?\d+)+}')
    res1 = pat.search(str1)
    
    if res1 is not None:
        #get a string like:RunIdList={-10513,0,-10629}
        str2 = res1.group()

        result["description"] = str2
        pat = re.compile(r'-\d+(,-?\d+)+')
        list = pat.search(str2)
        
        if list is not None:
            #get a string like:-10513,0,-10629
            runIds = list.group()

            #split runIds according ','
            items=runIds.split(',')

            #members' style in items is string,we need to change their style to integer
            for i in items:
                if i!='0':
                    runid=abs(string.atoi(i))
                    runIdList.append(runid)

            result["runIdList"] = runIdList

    return result
        
        
        
#get Boss version, runid, Entry number, JobOptions from root file
def getCommonInfo(dstfile):
    
    import subprocess
    p = subprocess.Popen(["bash", os.path.join(get_module_dir(), "get_info.sh"), dstfile], stdout=subprocess.PIPE,
                          stderr=subprocess.PIPE)
    output = p.communicate()[0].strip()
    pos = output.find("{")
    if pos >= 0:
      return eval( output[pos:] )

#get bossVer,eventNumber,dataType,fileSize,name,eventType,expNum,
#resonance,runH,runL,status,streamId,description
class DataAll(object):
    def __init__(self,dstfile):
        self.dstfile = dstfile
    
    def getAttributes(self):
        #store all attributes
        attributes = {}
        runIds = []
        
        if getFileSize(self.dstfile)<5000:
            print "Content of this file is null:",self.dstfile
            return "error"
        else:
            attributes = getCommonInfo(self.dstfile)
            #attributes["fileSize"] = getFileSize(self.dstfile)
            #set values of attribute status,streamId,Description,...
            attributes["status"] = 1
            attributes["streamId"] = 'stream0' 
            attributes["description"] = ''
            attributes["LFN"] = getLFN(self.dstfile)
            attributes["eventType"] = "all"
            attributes["count"] = 0

            #NOTE, MUST RESET the follow two metadata
            attributes["round"] = "round06" 
            attributes["resonance"] = "xyz_4360scan"

            #get runId from filename
            runId = splitLFN(attributes["LFN"],"all")

            #compare runid of rootfile with runid in filename
            if attributes["runId"] == runId:
                runIds.append(attributes["runId"])
                #set RunH=RunId and RunL=RunId
                attributes["runH"] = attributes["runId"]
                attributes["runL"] = attributes["runId"]

            else:
                print "runId of %s,in filename is %d,in rootfile is %d"%\
                    (self.dstfile,lfnInfo["runId"],attributes["runId"])
                return "error"

            del attributes["runId"]
            #attributes["jobOptions"] = ""
            return attributes

#get resonance,runL,runH,streamId,LFN from file name
#file name like jpsi2009_stream001_run9996_file7.dst 
#get bossVer,runL,runH,eventNum by reading information from rootfile
class Others(object):
    def __init__(self,dstfile):
        self.dstfile = dstfile
        
    def getAttributes(self):
        attributes = {}
        lfnInfo = {}
        runIds = []

        if getFileSize(self.dstfile)<5000:
            print "Content of this file is null:",self.dstfile
            return "error"
        else:
            #get bossVer,datatype,eventNum,runId(equal runL,runH)
            attributes = getCommonInfo(self.dstfile)
            #attributes["fileSize"] = getFileSize(self.dstfile)
            attributes["LFN"] = getLFN(self.dstfile)
            attributes["status"] = 1
            attributes["count"] = 0
            #NOTE, MUST RESET the follow metadatas
            attributes["round"] = "round02" 
            attributes["resonance"] = "jpsi"
            attributes["eventType"] = "inclusive"

            #get resonance,streamId,runL,runH in filename by calling splitLFN function
            lfnInfo = splitLFN(attributes["LFN"],"others")
            attributes["resonance"] = lfnInfo["resonance"]
            attributes["streamId"] = lfnInfo["streamId"]
            #if runL is equal to runH,this file only has one runId
            if lfnInfo["runL"] == lfnInfo["runH"]:
                #if runId in filename also is equal to runId in rootfile
                #print "a_runId",attributes["runId"],lfnInfo["runL"],type(attributes["runId"]),type(lfnInfo["runL"])
                if attributes["runId"] == lfnInfo["runL"]:
                    runIds.append(attributes["runId"])
                    attributes["runL"] = attributes["runId"]
                    attributes["runH"] = attributes["runId"]
                    attributes["description"] = "null"
                else:
                    print "Error %s:in the filename,runL = runH = %s,but runId in the root file is %s"\
                        %(self.dstfile,lfnInfo["runL"],attributes["runId"])
                    return "error"
            else:
                #this dst file has several runIds,get them from JobOptions by calling getRunIdList function
                result = getRunIdList(attributes["jobOptions"])
                if result is not None:
                    runH = max(result["runIdList"])
                    runL = min(result["runIdList"])
                    attributes["runL"] = runL 
                    attributes["runH"] = runH
                    attributes["description"] = result["description"]
                    
            
            del attributes["runId"]
            #attributes["jobOptions"] = ""
            return attributes


if __name__=="__main__":
   import time
   client = FileCatalogClient()
   start = time.time()
   obj = DataAll("/bes3fs/offline/data/664p01/xyz/4230/dst/130402/run_0032239_All_file001_SFO-1.dst")
   #obj = Others("/besfs2/offline/data/664-1/jpsi/09mc/dst/jpsi2009_stream001_run10319_file1.dst")
   result = obj.getAttributes()
   print time.time()-start
   import pprint
   pprint.pprint(result)




''' SEAccessTest

A test class to test the access to ses.

'''

import subprocess, re
from datetime import datetime
from DIRAC import S_OK, S_ERROR, gConfig


class SEAccessTest:
  ''' SEAccessTest
  '''

  def _getAccessParams( self, element ):
    '''
      get the access host and port for the specified se.
    '''

    _basePath = 'Resources/StorageElements'

    host = gConfig.getValue( '%s/%s/AccessProtocol.1/Host' % ( _basePath, element ), '' )
    if not host:
      host = gConfig.getValue( '%s/%s/GFAL2_XROOT/Host' % ( _basePath, element ), '' )
    port = gConfig.getValue( '%s/%s/AccessProtocol.1/Port' % ( _basePath, element ), '' )
    if not port:
      port = gConfig.getValue( '%s/%s/GFAL2_XROOT/Port' % ( _basePath, element ), '' )

    return S_OK( ( host, port ) )



""" ResourceManagementIHEPDB

  ResourceManagementIHEPDB for IHEPDIRAC.

"""

from datetime                             import datetime

from DIRAC                                import S_OK, S_ERROR
from DIRAC.Core.Base.DB                   import DB
from IHEPDIRAC.ResourceStatusSystem.Utilities import MySQLWrapper

__RCSID__ = '$Id: $'

class ResourceManagementIHEPDB( object ):
  """
    Class that defines the tables for the ResourceManagementIHEPDB on a python dictionary.
  """

  # Written PrimaryKey as list on purpose !!
  _tablesDB = {}
  _tablesDB[ 'StorageCache' ] = { 'Fields' :
                                 {
                                  'SE'            : 'VARCHAR(64) NOT NULL',
                                  'Occupied'      : 'BIGINT UNSIGNED NOT NULL DEFAULT 0',
                                  'Free'          : 'BIGINT UNSIGNED NOT NULL DEFAULT 0',
                                  'Usage'         : 'DOUBLE NOT NULL DEFAULT 0.0',
                                  'LastCheckTime' : 'DATETIME NOT NULL'
                                  },
                                 'PrimaryKey' : [ 'SE' ]
                                 }

  _tablesDB[ 'JobCache' ] = { 'Fields' :
                      {
                       'Site'          : 'VARCHAR(64) NOT NULL',
                       'MaskStatus'    : 'VARCHAR(32) NOT NULL',
                       'Efficiency'    : 'DOUBLE NOT NULL DEFAULT 0',
                       'Running'       : 'INTEGER NOT NULL DEFAULT 0',
                       'Waiting'       : 'INTEGER NOT NULL DEFAULT 0',
                       'Done'          : 'INTEGER NOT NULL DEFAULT 0',
                       'Failed'        : 'INTEGER NOT NULL DEFAULT 0',
                       'Completed'     : 'INTEGER NOT NULL DEFAULT 0',
                       'Stalled'       : 'INTEGER NOT NULL DEFAULT 0 ',
                       'Status'        : 'VARCHAR(16) NOT NULL',
                       'LastCheckTime' : 'DATETIME NOT NULL'
                      },
                      'PrimaryKey' : [ 'Site' ]
                                }

  _tablesDB[ 'WorkNodeCache' ] = { 'Fields' :
                              {
                               'Host'          : 'VARCHAR(128) NOT NULL',
                               'Site'          : 'VARCHAR(32) NOT NULL',
                               'Done'          : 'INTEGER NOT NULL DEFAULT 0',
                               'Failed'        : 'INTEGER NOT NULL DEFAULT 0',
                               'Efficiency'    : 'DOUBLE NOT NULL DEFAULT 0.0',
                               'LastCheckTime' : 'DATETIME NOT NULL'
                               },
                              'PrimaryKey' : [ 'Host' ]
                              }

  _tablesDB[ 'SAMResult' ] = { 'Fields' :
                     {
                       'ElementName'     : 'VARCHAR(64) NOT NULL',
                       'TestType'        : 'VARCHAR(16) NOT NULL',
                       'ElementType'     : 'VARCHAR(16) NOT NULL',
                       'Status'          : 'VARCHAR(8) NOT NULL',
                       'Log'             : 'MEDIUMTEXT NOT NULL',
                       'JobID'           : 'INTEGER NOT NULL',
                       'SubmissionTime'  : 'DATETIME NOT NULL',
#                       'CompletionTime'  : 'DATETIME NOT NULL DEFAULT "0000-00-00"',
                       'CompletionTime'  : 'DATETIME NOT NULL',
                       'ApplicationTime' : 'DOUBLE NOT NULL DEFAULT 0',
                       'LastCheckTime'   : 'DATETIME NOT NULL'
                     },
                     'PrimaryKey' : [ 'ElementName' , 'TestType' ]
                                }

  _tablesDB[ 'ResourceSAMStatus' ] = { 'Fields' :
                     {
                       'VO'            : 'VARCHAR(32) NOT NULL',
                       'ElementName'   : 'VARCHAR(64) NOT NULL',
                       'ElementType'   : 'VARCHAR(16) NOT NULL',
                       'Tests'         : 'VARCHAR(256) NOT NULL DEFAULT ""',
                       'Status'        : 'VARCHAR(8) NOT NULL DEFAULT ""',
                       'LastCheckTime' : 'DATETIME NOT NULL',
                     },
                     'PrimaryKey' : [ 'VO', 'ElementName' ]
                                }

  _tablesDB[ 'SiteSAMStatus' ] = { 'Fields' :
                     {
                       'VO'            : 'VARCHAR(32) NOT NULL',
                       'Site'          : 'VARCHAR(32) NOT NULL',
                       'SiteType'      : 'VARCHAR(8) NOT NULL',
                       'Status'        : 'VARCHAR(8) NOT NULL DEFAULT ""',
                       'CEStatus'      : 'VARCHAR(8) NOT NULL DEFAULT ""',
                       'SEStatus'      : 'VARCHAR(8) NOT NULL DEFAULT ""',
                       'LastCheckTime' : 'DATETIME NOT NULL',
                     },
                     'PrimaryKey' : [ 'VO', 'Site' ]
                                }

  _tablesLike  = {}
  _tablesLike[ 'SAMResultWithID' ] = { 'Fields' :
                     {
                       'ID'              : 'BIGINT UNSIGNED AUTO_INCREMENT NOT NULL',
                       'ElementName'     : 'VARCHAR(64) NOT NULL',
                       'TestType'        : 'VARCHAR(16) NOT NULL',
                       'ElementType'     : 'VARCHAR(16) NOT NULL',
                       'Status'          : 'VARCHAR(8) NOT NULL',
                       'Log'             : 'MEDIUMTEXT NOT NULL',
                       'JobID'           : 'INTEGER NOT NULL',
                       'SubmissionTime'  : 'DATETIME NOT NULL',
#                       'CompletionTime'  : 'DATETIME NOT NULL DEFAULT "0000-00-00"',
                       'CompletionTime'  : 'DATETIME NOT NULL',
                       'ApplicationTime' : 'DOUBLE NOT NULL DEFAULT 0',
                       'LastCheckTime'   : 'DATETIME NOT NULL'
                     },
                     'PrimaryKey' : [ 'ID' ]
                                }

  _tablesLike[ 'ResourceSAMStatusWithID' ] = { 'Fields' :
                     {
                       'ID'            : 'BIGINT UNSIGNED AUTO_INCREMENT NOT NULL',
                       'VO'            : 'VARCHAR(32) NOT NULL',
                       'ElementName'   : 'VARCHAR(64) NOT NULL',
                       'ElementType'   : 'VARCHAR(16) NOT NULL',
                       'Tests'         : 'VARCHAR(256) NOT NULL DEFAULT ""',
                       'Status'        : 'VARCHAR(8) NOT NULL DEFAULT ""',
                       'LastCheckTime' : 'DATETIME NOT NULL',
                     },
                     'PrimaryKey' : [ 'ID' ]
                                }

  _tablesLike[ 'SiteSAMStatusWithID' ] = { 'Fields' :
                     {
                       'ID'            : 'BIGINT UNSIGNED AUTO_INCREMENT NOT NULL',
                       'VO'            : 'VARCHAR(32) NOT NULL',
                       'Site'          : 'VARCHAR(32) NOT NULL',
                       'SiteType'      : 'VARCHAR(8) NOT NULL',
                       'Status'        : 'VARCHAR(8) NOT NULL DEFAULT ""',
                       'CEStatus'      : 'VARCHAR(8) NOT NULL DEFAULT ""',
                       'SEStatus'      : 'VARCHAR(8) NOT NULL DEFAULT ""',
                       'LastCheckTime' : 'DATETIME NOT NULL',
                     },
                     'PrimaryKey' : [ 'ID' ]
                                }

  _likeToTable = {
                   'SAMResultLog'             : 'SAMResultWithID',
                   'ResourceSAMStatusLog'     : 'ResourceSAMStatusWithID',
                   'SiteSAMStatusLog'         : 'SiteSAMStatusWithID',
                  }


  def __init__( self, mySQL = None ):
    '''
      Constructor, accepts any DB or mySQL connection, mostly used for testing
      purposes.
    '''
    self._tableDict = self.__generateTables()

    if mySQL is not None:
      self.database = mySQL
    else:
      self.database = DB( 'ResourceManagementIHEPDB',
                          'ResourceStatus/ResourceManagementIHEPDB' )

  ## SQL Methods ###############################################################

  def insert( self, params, meta ):
    '''
    Inserts args in the DB making use of kwargs where parameters such as
    the 'table' are specified ( filled automatically by the Client). Typically you
    will not pass kwargs to this function, unless you know what are you doing
    and you have a very special use case.

    :Parameters:
      **params** - `dict`
        arguments for the mysql query ( must match table columns ! ).

      **meta** - `dict`
        metadata for the mysql query. It must contain, at least, `table` key
        with the proper table name.

    :return: S_OK() || S_ERROR()
    '''

    utcnow = datetime.utcnow().replace( microsecond = 0 )

    # We force lastCheckTime to utcnow if it is not present on the params
    #if not( 'lastCheckTime' in params and not( params[ 'lastCheckTime' ] is None ) ):
    if 'lastCheckTime' in params and params[ 'lastCheckTime' ] is None:
      params[ 'lastCheckTime' ] = utcnow

    if 'dateEffective' in params and params[ 'dateEffective' ] is None:
      params[ 'dateEffective' ] = utcnow

    return MySQLWrapper.insert( self, params, meta )

  def update( self, params, meta ):
    '''
    Updates row with values given on args. The row selection is done using the
    default of MySQLMonkey ( column.primary or column.keyColumn ). It can be
    modified using kwargs. The 'table' keyword argument is mandatory, and
    filled automatically by the Client. Typically you will not pass kwargs to
    this function, unless you know what are you doing and you have a very
    special use case.

    :Parameters:
      **params** - `dict`
        arguments for the mysql query ( must match table columns ! ).

      **meta** - `dict`
        metadata for the mysql query. It must contain, at least, `table` key
        with the proper table name.

    :return: S_OK() || S_ERROR()
    '''

    # We force lastCheckTime to utcnow if it is not present on the params
    #if not( 'lastCheckTime' in params and not( params[ 'lastCheckTime' ] is None ) ):
    if 'lastCheckTime' in params and params[ 'lastCheckTime' ] is None:
      params[ 'lastCheckTime' ] = datetime.utcnow().replace( microsecond = 0 )

    return MySQLWrapper.update( self, params, meta )

  def select( self, params, meta ):
    '''
    Uses arguments to build conditional SQL statement ( WHERE ... ). If the
    sql statement desired is more complex, you can use kwargs to interact with
    the MySQL buildCondition parser and generate a more sophisticated query.

    :Parameters:
      **params** - `dict`
        arguments for the mysql query ( must match table columns ! ).

      **meta** - `dict`
        metadata for the mysql query. It must contain, at least, `table` key
        with the proper table name.

    :return: S_OK() || S_ERROR()
    '''
    return MySQLWrapper.select( self, params, meta )

  def delete( self, params, meta ):
    """
    Uses arguments to build conditional SQL statement ( WHERE ... ). If the
    sql statement desired is more complex, you can use kwargs to interact with
    the MySQL buildCondition parser and generate a more sophisticated query.
    There is only one forbidden query, with all parameters None ( this would
    mean a query of the type DELETE * from TableName ). The usage of kwargs
    is the same as in the get function.

    :Parameters:
      **params** - `dict`
        arguments for the mysql query ( must match table columns ! ).

      **meta** - `dict`
        metadata for the mysql query. It must contain, at least, `table` key
        with the proper table name.

    :return: S_OK() || S_ERROR()
    """
    return MySQLWrapper.delete( self, params, meta )

  ## Extended SQL methods ######################################################

  def addOrModify( self, params, meta ):
    '''
    Using the PrimaryKeys of the table, it looks for the record in the database.
    If it is there, it is updated, if not, it is inserted as a new entry.

    :Parameters:
      **params** - `dict`
        arguments for the mysql query ( must match table columns ! ).

      **meta** - `dict`
        metadata for the mysql query. It must contain, at least, `table` key
        with the proper table name.

    :return: S_OK() || S_ERROR()
    '''

    selectQuery = self.select( params, meta )
    if not selectQuery[ 'OK' ]:
      return selectQuery

    isUpdate = False

    if selectQuery[ 'Value' ]:

      # Pseudo - code
      # for all column not being PrimaryKey and not a time column:
      #   if one or more column different than params if not None:
      #     we update dateTime as well

      columns = selectQuery[ 'Columns' ]
      values  = selectQuery[ 'Value' ]

      if len( values ) != 1:
        return S_ERROR( 'More than one value returned on addOrModify, please report !!' )

      selectDict = dict( zip( columns, values[ 0 ] ) )

      newDateEffective = None

      for key, value in params.items():
        if key in ( 'lastCheckTime', 'dateEffective' ):
          continue

        if value is None:
          continue

        if value != selectDict[ key[0].upper() + key[1:] ]:
          newDateEffective = datetime.utcnow().replace( microsecond = 0 )
          break

      if 'dateEffective' in params:
        params[ 'dateEffective' ] = newDateEffective

      userQuery = self.update( params, meta )
      isUpdate  = True

    else:
      userQuery = self.insert( params, meta )

    # This part only applies to PolicyResult table
    logResult = self._logRecord( params, meta, isUpdate )
    if not logResult[ 'OK' ]:
      return logResult

    return userQuery

  # FIXME: this method looks unused. Maybe can be removed from the code.
  def addIfNotThere( self, params, meta ):
    '''
    Using the PrimaryKeys of the table, it looks for the record in the database.
    If it is not there, it is inserted as a new entry.

    :Parameters:
      **params** - `dict`
        arguments for the mysql query ( must match table columns ! ).

      **meta** - `dict`
        metadata for the mysql query. It must contain, at least, `table` key
        with the proper table name.

    :return: S_OK() || S_ERROR()
    '''

    selectQuery = self.select( params, meta )
    if not selectQuery[ 'OK' ]:
      return selectQuery

    if selectQuery[ 'Value' ]:
      return selectQuery

    return self.insert( params, meta )

  ## Auxiliar methods ##########################################################

  def getTable( self, tableName ):
    '''
      Returns a table dictionary description given its name
    '''
    if tableName in self._tableDict:
      return S_OK( self._tableDict[ tableName ] )

    return S_ERROR( '%s is not on the schema' % tableName )

  def getTablesList( self ):
    '''
      Returns a list of the table names in the schema.
    '''
    return S_OK( self._tableDict.keys() )

  ## Protected methods #########################################################

  def _checkTable( self ):
    '''
      Method used by database tools to write the schema
    '''
    return self.__createTables()

  def _logRecord( self, params, meta, isUpdate ):
    '''
      Method that records every change on a LogTable.
    '''

    tables = [ 'SAMResult', 'ResourceSAMStatus', 'SiteSAMStatus' ]
    if not ( 'table' in meta and meta[ 'table' ] in tables ):
      return S_OK()

    if isUpdate:

      updateRes = self.select( params, meta )
      if not updateRes[ 'OK' ]:
        return updateRes

      params = dict( zip( updateRes[ 'Columns' ], updateRes[ 'Value' ][ 0 ] ))

    meta[ 'table' ] += 'Log'

    logRes = self.insert( params, meta )

    return logRes


  ## Private methods ###########################################################

  def __createTables( self, tableName = None ):
    '''
      Writes the schema in the database. If no tableName is given, all tables
      are written in the database. If a table is already in the schema, it is
      skipped to avoid problems trying to create a table that already exists.
    '''

    # Horrible SQL here !!
    tablesCreatedRes = self.database._query( "show tables" )
    if not tablesCreatedRes[ 'OK' ]:
      return tablesCreatedRes
    tablesCreated = [ tableCreated[0] for tableCreated in tablesCreatedRes[ 'Value' ] ]

    tables = {}
    if tableName is None:
      tables.update( self._tableDict )

    elif tableName in self._tableDict:
      tables = { tableName : self._tableDict[ tableName ] }

    else:
      return S_ERROR( '"%s" is not a known table' % tableName )

    for tableName in tablesCreated:
      if tableName in tables:
        del tables[ tableName ]

    res = self.database._createTables( tables )
    if not res[ 'OK' ]:
      return res

    # Human readable S_OK message
    if res[ 'Value' ] == 0:
      res[ 'Value' ] = 'No tables created'
    else:
      res[ 'Value' ] = 'Tables created: %s' % ( ','.join( tables.keys() ) )
    return res

  def __generateTables( self ):
    '''
      Method used to transform the class variables into instance variables,
      for safety reasons.
    '''

    # Avoids copying object.
    tables = {}
    tables.update( self._tablesDB )

    for tableName, tableLike in self._likeToTable.items():

      tables[ tableName ] = self._tablesLike[ tableLike ]

    return tables

################################################################################
#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF



''' CEAccessTest

A test class to test the access to ces.

'''

from DIRAC import S_OK, S_ERROR, gConfig


class CEAccessTest:
  ''' CEAccessTest
  '''

  def _getAccessParams( self, element ):
    '''
      get the access host and port for the specified ce.
    '''

    _basePath = 'Resources/Sites'

    domains = gConfig.getSections( _basePath )
    if not domains[ 'OK' ]:
      return domains
    domains = domains[ 'Value' ]

    for domain in domains:
      sites = gConfig.getSections( '%s/%s' % ( _basePath, domain ) )
      if not sites[ 'OK' ]:
        return sites
      sites = sites[ 'Value' ]

      for site in sites:
        ces = gConfig.getValue( '%s/%s/%s/CE' % ( _basePath, domain, site ), '' ).split(',')
        ces = map(lambda str : str.strip(), ces)

        if element in ces:
          host = gConfig.getValue('%s/%s/%s/CEs/%s/SSHHost' % ( _basePath, domain, site, element ))
          cetype = gConfig.getValue('%s/%s/%s/CEs/%s/CEType' % ( _basePath, domain, site, element ))
          if host:
            idx = host.find('/')
            if idx != -1: host = host[ 0 : idx ]
            return S_OK((host, 22))
          elif cetype == 'CREAM':
            return S_OK((element, 8443))
          elif cetype == 'HTCondorCE':
            return S_OK((element, 9619))
          elif cetype == 'ARC':
            return S_OK((element, 2135))
          else:
            return S_OK((element, 8443))

    return S_ERROR('%s is not a vaild CE.' % element)



import os

from DIRAC import gLogger
from DIRAC.Interfaces.API.Dirac import Dirac

from IHEPDIRAC.Badger.private.output.getfile.GetFile import GetFile

class DfcGetFile(GetFile):
  def __init__(self):
    super(DfcGetFile, self).__init__()

  def _available(self):
    return True

  def _downloadSingleFile(self, remotePath, localPath):
    gLogger.debug('getfile from %s to %s' % (remotePath, localPath))
    dirac = Dirac()
    result = dirac.getFile(remotePath, os.path.dirname(localPath))
    return result['OK']



""" CVMFSTest

  A test class to test CVMFS is ok or not.

"""


from IHEPDIRAC.ResourceStatusSystem.SAM.SAMTest.CEBaseTest import CEBaseTest


__RCSID__ = '$Id: $'


class CVMFSTest( CEBaseTest ):
  """
    CVMFSTest is used  to test whether CVMFS is fine to run jobs.
  """

  def __init__( self, args = None, apis = None ):
    super( CVMFSTest, self ).__init__( args, apis )


  @staticmethod
  def _judge( log ):
    """
      judge the CVMFS test status.
    """

    if log.find( 'Standard Error' ) != -1:
      return 'Bad'
    else:
      return 'OK'
